/**
 * @Filename SocRobXMLDoc.hpp
 * @Description SocRobXMLDoc class, used to parse a .xml file. Create a tree of SocRobXMLNode classes.
 * @Status Finished
 * @Version $Id: SocRobXMLDoc.hpp
 * @Maintainer Nelson Ramos (nmsra@mega.ist.utl.pt)
 */

#ifndef _SOCROBXML_SOCROBXMLDOC_
#define _SOCROBXML_SOCROBXMLDOC_

#include <string>

#include "SocRobXMLNode.hpp"



typedef struct _xmlDoc xmlDoc;
typedef xmlDoc* xmlDocPtr;
typedef struct _xmlNode xmlNode;
typedef xmlNode* xmlNodePtr;



using namespace std;



/**
 * @Class SocRobXMLDoc SocRobXMLDoc.hpp "SocRobXMLDoc.hpp"
 * Class to parse xml files.
 * @Author Nelson Ramos
 */



class SocRobXMLDoc
{
  public:
  
    /**
     * Constructor.
     * This method creates a new SocRobXMLDoc Class
     * @Author Nelson Ramos
     */
    SocRobXMLDoc (string logPath);
    
    
    /**
     * Destructor.
     * This method makes deinitializations for objects of this class.
     * @Author Nelson Ramos
     */
    ~SocRobXMLDoc();
    
    /**
     * Parse XML file. Create a tree of SocRobXMLNode classes.
     * @Argument name - ?
     * @Returns bool false if not sucessfull
     * @Author Nelson Ramos
     */
    bool parseFile (string name);
    
    /**
     * Get the root node of tree build by parseFile.
     * @Returns SocRobXMLNode, rootNode
     * @Author Nelson Ramos
     */
    void getRootNode (SocRobXMLNode* node);
    
  protected:
  
  private:
  
    xmlDocPtr docPtr_; ///< libxml2 stuff
    xmlNodePtr curPtr_; ///< libxml2 stuff
    
    string logPath_; ///< Log folder
    
    // These are used to prevent copying the class object
    SocRobXMLDoc (const SocRobXMLDoc&);
    SocRobXMLDoc& operator= (const SocRobXMLDoc&);
};

#endif
