/**
 * @Filename SocRobXMLNode.hpp
 * @Description SocRobXMLNode class, used to build a tree of xml nodes.
 * @Status Finished
 * @Version $Id: SocRobXMLNode.hpp
 * @Maintainer Nelson Ramos (nmsra@mega.ist.utl.pt)
 */

#ifndef _SOCROBXML_SOCROBXMLNODE_
#define _SOCROBXML_SOCROBXMLNODE_

#include <string>



typedef struct _xmlDoc xmlDoc;
typedef xmlDoc* xmlDocPtr;
typedef struct _xmlNode xmlNode;
typedef xmlNode* xmlNodePtr;



using namespace std;



/**
 * @Class SocRobXMLNode SocRobXMLNode.hpp "SocRobXMLNode.hpp"
 * Class for create a tree of XML Nodes. This is useful for simple components configuration read, or define predicates for example.
 * @Author Nelson Ramos
 */



class SocRobXMLNode
{
  public:
  
    /**
     * Constructor.
     * This method creates a new SocRobXML Node Class, but don't use this! Use SocRobXMLNode(xmlDocPtr doc, xmlNodePtr node).
     * @Author Nelson Ramos
     */
    SocRobXMLNode() : docLibxml_ (0), nodeLibxml_ (0), nodeLibxmlNextSubAtribute_ (0) {};
    
    /**
     * Constructor.
     * This method creates a new SocRobXML Node Class. This constructor is for SocRobXMLDoc use.
     * @Argument doc - ?
     * @Argument node - ?
     * @Author Nelson Ramos
     */
    SocRobXMLNode (xmlDocPtr doc, xmlNodePtr node);
    
    /**
     * Destructor.
     * This method makes deinitializations for objects of this class.
     * @Author Nelson Ramos
     */
    ~SocRobXMLNode();
    
    /**
     * This method is usefull when the empty constructor was used.
     */
    void fillXMLNode (xmlDocPtr doc, xmlNodePtr node);
    
    /**
     * Copy the reference of subnode name to node given as atribute.
     * This method get the first subNode with the name passed as atribute.
     * @Argument name - ?
     * @Argument node - ?
     * @Returns bool subNode exists or not.
     * @Author Nelson Ramos
     */
    bool getSubNode (string name, SocRobXMLNode* node);
    
    /**
     * Copy the reference of next subnode to node given as atribute.
     * This method know what is the last returned subNode, return the next.
     * @Argument node - ?
     * @Returns bool false if don't exist more subNodes.
     * @Author Nelson Ramos
     */
    bool getNextSubNode (SocRobXMLNode* node);
    
    /**
     * Copy the reference of next subnode with name to node given as atribute.
     * This method know what is the last returned subNode, return the next.
     * @Argument name - ?
     * @Argument node - ?
     * @Returns bool false if don't exist more subNodes.
     * @Author Nelson Ramos
     */
    bool getNextSubNode (string name, SocRobXMLNode* node);
    
    /**
     * Save the value of an atribute in passed reference.
     * Careful! You must know that this atribute is integer!
     * @Argument name - ?
     * @Argument atribPtr - ?
     * @Returns bool false if don't exist this atribute name.
     * @Author Nelson Ramos
     */
    bool getAtributeInt (string name, int& atribPtr);
    
    /**
     * Save the value of the next atribute in passed reference.
     * Consecutive calls of this method will iterate the atribute
     * list.
     * Careful! You must know that this atribute is integer!
     * @Argument name - ?
     * @Argument atribPtr - ?
     * @Returns bool false if don't exist this atribute name.
     * @Author Nelson Ramos
     */
    bool getNextAtributeInt (string name, int& atribPtr);
    
    
    /**
     * Save the value of an atribute in passed reference.
     * Careful! You must know that this atribute is bool!
     * @Argument name - ?
     * @Argument atribPtr - ?
     * @Returns bool false if don't exist this atribute name.
     * @Author Nelson Ramos
     */
    bool getAtributeBool (string name, bool& atribPtr);
    
    /**
     * Save the value of the next atribute in passed reference.
     * Consecutive calls of this method will iterate the atribute
     * list.
     * Careful! You must know that this atribute is bool!
     * @Argument name - ?
     * @Argument atribPtr - ?
     * @Returns bool false if don't exist this atribute name.
     * @Author Nelson Ramos
     */
    bool getNextAtributeBool (string name, bool& atribPtr);
    
    
    /**
     * Save the value of an atribute in passed reference.
     * Careful! You must know that this atribute is float!
     * @Argument name - ?
     * @Argument atribPtr - ?
     * @Returns bool false if don't exist this atribute name.
     * @Author Nelson Ramos
     */
    bool getAtributeFloat (string name, float& atribPtr);
    
    /**
     * Save the value of the next atribute in passed reference.
     * Consecutive calls of this method will iterate the atribute
     * list.
     * Careful! You must know that this atribute is float!
     * @Argument name - ?
     * @Argument atribPtr - ?
     * @Returns bool false if don't exist this atribute name.
     * @Author Nelson Ramos
     */
    bool getNextAtributeFloat (string name, float& atribPtr);
    
    /**
     * Save the value of an atribute in passed reference.
     * Careful! You must know that this atribute is string!
     * @Argument name - ?
     * @Argument atribPtr - ?
     * @Returns bool false if don't exist this atribute name.
     * @Author Nelson Ramos
     */
    bool getAtributeString (string name, string& atribPtr);
    
    
    /**
     * Save the value of the next atribute in passed reference.
     * Consecutive calls of this method will iterate the atribute
     * list.
     * Careful! You must know that this atribute is string!
     * @Argument name - ?
     * @Argument atribPtr - ?
     * @Returns bool false if don't exist this atribute name.
     * @Author Nelson Ramos
     */
    bool getNextAtributeString (string name, string& atribPtr);
    
    
    /**
     * Return the name of this node.
     * @Returns string Return the name of this node.
     * @Author Nelson Ramos
     */
    string getName();
    
  protected:
  
  private:
  
    xmlDocPtr docLibxml_; ///< libxml2 stuff
    xmlNodePtr nodeLibxml_; ///< libxml2 stuff
    xmlNodePtr nodeLibxmlNextSubAtribute_; ///< libxml2 stuff
    
    // These are used to prevent copying the supervisor
    SocRobXMLNode (const SocRobXMLNode&);
    SocRobXMLNode& operator= (const SocRobXMLNode&);
};

#endif
