/**
 * @Filename XmlAttribute.hpp
 * @Description XmlAttribute definition file.
 * @Status Implementing
 * @Version $Id: XmlAttribute.hpp 5913 2013-02-10 12:15:21Z jreis $
 * @Maintainer Nelson Ramos (nmsra@mega.ist.utl.pt)
 */

#ifndef __XML_XMLATTRIBUTE_H
#define __XML_XMLATTRIBUTE_H



namespace mermaid
{
  namespace support
  {
    namespace xml
    {
      class XmlAttribute;
    }
  }
}

#include <string>

#include <boost/shared_ptr.hpp>



typedef struct _xmlAttr xmlAttr;
typedef xmlAttr* xmlAttrPtr;



namespace mermaid
{
  namespace support
  {
    namespace xml
    {
      using std::string;
      
      using boost::shared_ptr;
      
      /**
      * @Class XmlAttribute XmlAttribute.hpp "XmlAttribute.hpp"
      * @Description Class representing a XML element's attribute.
      * @Author Marco Barbosa
      */
      class XmlAttribute
      {
          friend class XmlElement;
          
        public:
          /**
          * @Description XmlAttribute name getter
          * @Returns string XmlAttribute's name
          */
          const string getName() const;
          
          /**
          * @Description XmlAttribute value getter
          * @Returns string XmlAttribute's value
          */
          const string getValue() const;
          
        private:
          /**
          * @Description Constructor
          * @Description Builds a new XmlAttribute
          * @Argument attr libxml2 attribute node pointer
          */
          XmlAttribute (xmlAttrPtr attr);
          
          string name_;
          string value_;
      }; // XmlAttribute
    } // namespace xml
  } // namespace support
} // namespace mermaid

#endif
