/**
 * @Filename XmlAttributeVector.hpp
 * @Description XmlAttributeVector class definition.
 * @Status Implementing
 * @Version $Id: XmlAttributeVector.hpp 5913 2013-02-10 12:15:21Z jreis $
 * @Maintainer
 */

#ifndef __XML_XMLATTRIBUTEVECTOR_H_
#define __XML_XMLATTRIBUTEVECTOR_H_



namespace mermaid
{
  namespace support
  {
    namespace xml
    {
      class XmlAttributeVector;
    }
  }
}

#include <string>
#include <vector>

#include <boost/shared_ptr.hpp>

#include "XmlAttribute.hpp"



namespace mermaid
{
  namespace support
  {
    namespace xml
    {
      using std::string;
      using std::vector;
      
      using boost::shared_ptr;
      
      /**
       * @Class XmlAttributeVector XmlAttributeVector.hpp "XmlAttributeVector.hpp"
       * @Description Class representing an XML Attribute vector.
       * @Author Marco Barbosa
       */
      class XmlAttributeVector : public vector< shared_ptr<XmlAttribute> >
      {
        public:
          /**
           * @Description Generates an XmlAttributeVector that is a subset of
           * @Description the current one. The XmlAttributes inside the
           * @Description resulting XmlAttributeVector have their name equal
           * @Description to the string passed as argument.
           * @Argument name The name of the desired XmlAttributes
           * @Returns The resulting XmlAttributeVector
           */
          XmlAttributeVector getAttributesWithName (string name);
          
          /**
           * @Description Gets the first XmlAttribute that has the name
           * @Description equal to the string passed as argument.
           * @Argument name The name of the desired XmlAttribute
           * @Returns The desired XmlAttribute or NULL if it doesn't exist
           */
          shared_ptr<XmlAttribute> getFirstAttributeWithName (string name);
      }; // class XmlElementVector
    } // namespace xml
  } // namespace support
} // namespace mermaid

#endif
