/**
 * @Filename XmlCharData.hpp
 * @Description XmlCharData declaration file.
 * @Status Implementing
 * @Version $Id: XmlCharData.hpp 5913 2013-02-10 12:15:21Z jreis $
 * @Maintainer Nelson Ramos (nmsra@mega.ist.utl.pt)
 */

#ifndef __XML_XMLCHARDATA_H_
#define __XML_XMLCHARDATA_H_



namespace mermaid
{
  namespace support
  {
    namespace xml
    {
      class XmlCharData;
    }
  }
}

#include <string>

#include <boost/shared_ptr.hpp>

#include "XmlItem.hpp"



namespace mermaid
{
  namespace support
  {
    namespace xml
    {
      using std::string;
      
      using boost::shared_ptr;
      
      typedef vector<shared_ptr<XmlCharData> > XmlCharDataVector;
      
      /**
      * @Class XmlCharData XmlCharData.h "XmlCharData.h"
      * @Description Class that holds CharData from XML files.
      * @Description Inherits from string.
      * @Author Marco Barbosa
      */
      class XmlCharData : public XmlItem
      {
        public:
          /**
          * Constructor
          * Creates an empty XmlCharData instance.
          */
          XmlCharData();
          
          virtual ~XmlCharData() {}
          
          /**
          * Constructor
          * Creates an XmlCharData instance from a C-like string.
          * @Argument c C-like string
          */
          XmlCharData (const char* c);
          
          virtual XmlCharData* clone();
          
          /**
          * Constructor
          * Creates an XmlCharData instance from a string.
          * @Argument s string
          */
          XmlCharData (string s);
          
          /**
          * Copy constructor.
          */
          XmlCharData (const XmlCharData& c);
          
          /**
          * Checks if instance is filled only with blank characters.
          * @Returns TRUE if instance is filled only with blanks,
          * FALSE otherwise.
          */
          bool isBlank() const;
          
          /**
          * Removes blank characters from the beginning and end.
          */
          void removeEndBlanks();
          
          /**
          * @Description Method to convert value to string
          */
          virtual operator string() const;
          
          /**
           * @Description Method to check if XmlItem is of type XmlCharData
           * @Returns True
           */
          virtual bool isXmlCharData() const;
          
        private:
          string string_;
      }; // class XmlCharData
    } // namespace xml
  } // namespace support
} // namespace mermaid

#endif
