/**
 * @Filename XmlElementVector.hpp
 * @Description XmlElementVector class definition.
 * @Status Implementing
 * @Version $Id: XmlElementVector.hpp 5913 2013-02-10 12:15:21Z jreis $
 * @Maintainer Nelson Ramos (nmsra@mega.ist.utl.pt)
 */



// XmlElementVector is used in XmlElement, so it needs to be defined before.
// Including this here will cause XmlElementVector to be defined first.
#include "XmlElement.hpp"

#ifndef __XML_XMLELEMENTVECTOR_H_
#define __XML_XMLELEMENTVECTOR_H_



namespace mermaid
{
  namespace support
  {
    namespace xml
    {
      class XmlElementVector;
    }
  }
}

#include <string>

#include <boost/shared_ptr.hpp>



namespace mermaid
{
  namespace support
  {
    namespace xml
    {
      using std::string;
      
      using boost::shared_ptr;
      
      /**
      * @Class XmlElementVector XmlElementVector.hpp "XmlElementVector.hpp"
      * @Description Class representing an XML Element vector.
      * @Author Marco Barbosa
      */
      class XmlElementVector : public vector<shared_ptr<XmlElement> >
      {
        public:
          /**
          * @Description Generates an XmlElementVector that is a subset of
          * @Description the current one. The XmlElements inside the
          * @Description resulting XmlElementVector have their name equal
          * @Description to the string passed as argument.
          * @Argument name The name of the desired XmlElements
          * @Returns The resulting XmlElementVector
          */
          XmlElementVector getElementsWithName (string name);
          
          /**
          * @Description Gets the first XmlElement that has the name
          * @Description equal to the string passed as argument.
          * @Argument name The name of the desired XmlElement
          * @Returns The desired XmlElement or NULL if it doesn't exist
          */
          shared_ptr<XmlElement> getFirstElementWithName (string name);
      }; // class XmlElementVector
    } // namespace xml
  } // namespace support
} // namespace mermaid

#endif
