/**
 * @Filename XmlItem.hpp
 * @Description XmlItem definition file.
 * @Status Implementing
 * @Version $Id: XmlItem.hpp 5913 2013-02-10 12:15:21Z jreis $
 * @Maintainer Nelson Ramos (nmsra@mega.ist.utl.pt)
 */

#ifndef __XML_XMLITEM_H_
#define __XML_XMLITEM_H_



namespace mermaid
{
  namespace support
  {
    namespace xml
    {
      class XmlItem;
    }
  }
}

#include <string>
#include <vector>




namespace mermaid
{
  namespace support
  {
    namespace xml
    {
      using std::string;
      using std::vector;
      
      
      typedef vector<XmlItem*> XmlItemVector; ///< Vector of XmlItem's
      
      /**
      * @Class XmlItem XmlItem.hpp "XmlItem.hpp"
      * @Description Class representing an XML item.
      * @Author Marco Barbosa
      */
      class XmlItem
      {
        public:
          virtual ~XmlItem() {}
          
          /**
          * @Description Method to check if XmlItem is of type XmlElement
          * @Returns True if XmlItem is of type XmlElement, false otherwise
          */
          virtual bool isXmlElement() const;
          
          /**
          * @Description Method to check if XmlItem is of type XmlCharData
          * @Returns True if XmlItem is of type XmlCharData, false otherwise
          */
          virtual bool isXmlCharData() const;
          
          /**
          * @Description String cast operator
          */
          virtual operator string() const = 0;
      }; // XmlItem
    } // namespace xml
  } // namespace support
} // namespace mermaid

#endif
