/**
 * @Filename SocRobXMLDoc.cpp
 * @Description Implementation of the SocRobXMLDoc class
 * @Status Finished
 * @Version $Id: SocRobXMLDoc.cpp
 * @Maintainer Nelson Ramos (nmsra@mega.ist.utl.pt)
 */

#include <string>

#include "SocRobXMLDoc.hpp"
#include "SocRobXMLNode.hpp"

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include <ros/console.h>



SocRobXMLDoc::SocRobXMLDoc (string logPath) :
  docPtr_ (0), curPtr_ (0), logPath_ (logPath)
{
}



SocRobXMLDoc::~SocRobXMLDoc()
{
  if (docPtr_) {
    xmlFreeDoc (docPtr_);
  }
  
  // xmlCleanupParser(); <- Read the docs and you'll understand.
  // I leave this here as a memory for more than 20 hrs of my life lost in debugging.
}



bool SocRobXMLDoc::parseFile (string name)
{
  docPtr_ = xmlParseFile (name.data());
  if (docPtr_ == NULL) {
    ROS_ERROR_STREAM_NAMED ("SocRobXMLDoc", "parseFile: Document \"" << name << "\" not parsed successfully.");
    return false;
  }
  
  curPtr_ = xmlDocGetRootElement (docPtr_);
  if (curPtr_ == NULL) {
    ROS_ERROR_STREAM_NAMED ("SocRobXMLDoc", "parseFile: Empty document \"" << name << "\".");
    xmlFreeDoc (docPtr_);
    return false;
  }
  
  return true;
}



void SocRobXMLDoc::getRootNode (SocRobXMLNode* node)
{
  node->fillXMLNode (docPtr_, curPtr_);
}
