/**
 * @Filename SocRobXMLNode.cpp
 * @Description Implementation of the SocRobXMLNode class
 * @Status Finished
 * @Version $Id: SocRobXMLNode.cpp
 * @Maintainer Nelson Ramos (nmsra@mega.ist.utl.pt)
 */

#include <string>
#include <iostream>

#include "SocRobXMLNode.hpp"
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <stdlib.h>

#include <boost/algorithm/string.hpp>
#include <boost/lexical_cast.hpp>



using namespace boost;



SocRobXMLNode::SocRobXMLNode (xmlDocPtr doc, xmlNodePtr node) :
  docLibxml_ (doc), nodeLibxml_ (node),
  nodeLibxmlNextSubAtribute_ (node->xmlChildrenNode)
{
}



void SocRobXMLNode::fillXMLNode (xmlDocPtr doc, xmlNodePtr node)
{
  nodeLibxml_ = node;
  docLibxml_ = doc;
  nodeLibxmlNextSubAtribute_ = nodeLibxml_->xmlChildrenNode;
}



SocRobXMLNode::~SocRobXMLNode()
{
}



bool SocRobXMLNode::getSubNode (string name, SocRobXMLNode* node)
{
  xmlNodePtr cur = nodeLibxml_->xmlChildrenNode;
  
  while (cur != NULL) {
    if ( (!xmlStrcmp (cur->name, (const xmlChar*) name.data()))) {
      if (node == NULL) {
        return false;
      }
      node->fillXMLNode (docLibxml_, cur);
      
      return true;
    }
    
    cur = cur->next;
  }
  
  return false;
}



bool SocRobXMLNode::getNextSubNode (SocRobXMLNode* node)
{
  if (nodeLibxmlNextSubAtribute_ == NULL) {
    return false;
  }
  else {
    nodeLibxmlNextSubAtribute_ = nodeLibxmlNextSubAtribute_->next;
    if (nodeLibxmlNextSubAtribute_ == NULL) {
      return false;
    }
    else {
      if (node == NULL) {
        return false;
      }
      node->fillXMLNode (docLibxml_, nodeLibxmlNextSubAtribute_);
      return true;
    }
  }
}



bool SocRobXMLNode::getNextSubNode (string name, SocRobXMLNode* node)
{
  if (nodeLibxmlNextSubAtribute_ == NULL) {
    return false;
  }
  
  while ( (nodeLibxmlNextSubAtribute_ = nodeLibxmlNextSubAtribute_->next) != NULL) {
    if ( (!xmlStrcmp (nodeLibxmlNextSubAtribute_->name, (const xmlChar*) name.data()))) {
      if (node == NULL) {
        return false;
      }
      node->fillXMLNode (docLibxml_, nodeLibxmlNextSubAtribute_);
      
      return true;
    }
  }
  return false;
}



bool SocRobXMLNode::getAtributeInt (string name, int& atrib)
{
  xmlChar* key;
  xmlNodePtr cur = nodeLibxml_->xmlChildrenNode;
  
  while (cur != NULL) {
    if ( (!xmlStrcmp (cur->name, (const xmlChar*) name.data()))) {
      key = xmlNodeListGetString (docLibxml_, cur->xmlChildrenNode, 1);
      atrib = atoi ( (char*) key);
      xmlFree (key);
      return true;
    }
    
    cur = cur->next;
  }
  return false;
}



bool SocRobXMLNode::getNextAtributeInt (string name, int& atrib)
{
  xmlChar* key;
  if (nodeLibxmlNextSubAtribute_ == NULL) {
    return false;
  }
  while ( (nodeLibxmlNextSubAtribute_ = nodeLibxmlNextSubAtribute_->next) != NULL) {
    if ( (!xmlStrcmp (nodeLibxmlNextSubAtribute_->name, (const xmlChar*) name.data()))) {
      key = xmlNodeListGetString (docLibxml_, nodeLibxmlNextSubAtribute_->xmlChildrenNode, 1);
      atrib = atoi ( (char*) key);
      xmlFree (key);
      return true;
    }
  }
  return false;
}



bool SocRobXMLNode::getAtributeBool (string name, bool& atrib)
{
  xmlChar* key;
  xmlNodePtr cur = nodeLibxml_->xmlChildrenNode;
  
  while (cur != NULL) {
    if ( (!xmlStrcmp (cur->name, (const xmlChar*) name.data()))) {
      key = xmlNodeListGetString (docLibxml_, cur->xmlChildrenNode, 1);
      
      // never use this type of cast without adult supervision
      string k = string (reinterpret_cast<char*> (key));
      xmlFree (key);
      
      trim (k);
      to_lower (k);
      if (k == "") {
        atrib = false;
        return true;
      }
      try {
        float a = lexical_cast<float> (k);
        atrib = a != 0;
        return true;
      }
      catch (bad_lexical_cast) {}
      if (k == "false") {
        atrib = false;
        return true;
      }
      
      atrib = true;
      return true;
    }
    
    cur = cur->next;
  }
  return false;
}



bool SocRobXMLNode::getNextAtributeBool (string name, bool& atrib)
{
  xmlChar* key;
  if (nodeLibxmlNextSubAtribute_ == NULL) {
    return false;
  }
  while ( (nodeLibxmlNextSubAtribute_ = nodeLibxmlNextSubAtribute_->next) != NULL) {
    if ( (!xmlStrcmp (nodeLibxmlNextSubAtribute_->name, (const xmlChar*) name.data()))) {
      key = xmlNodeListGetString (docLibxml_, nodeLibxmlNextSubAtribute_->xmlChildrenNode, 1);
      
      // never use this type of cast without adult supervision
      string k = string (reinterpret_cast<char*> (key));
      xmlFree (key);
      
      trim (k);
      to_lower (k);
      if (k == "") {
        atrib = false;
        return true;
      }
      try {
        float a = lexical_cast<float> (k);
        atrib = a != 0;
        return true;
      }
      catch (bad_lexical_cast) {}
      if (k == "false") {
        atrib = false;
        return true;
      }
      
      atrib = true;
      return true;
    }
  }
  return false;
}



bool SocRobXMLNode::getAtributeFloat (string name, float& atrib)
{
  xmlChar* key;
  xmlNodePtr cur = nodeLibxml_->xmlChildrenNode;
  
  while (cur != NULL) {
    if ( (!xmlStrcmp (cur->name, (const xmlChar*) name.data()))) {
      key = xmlNodeListGetString (docLibxml_, cur->xmlChildrenNode, 1);
      atrib = atof ( (char*) key);
      xmlFree (key);
      return true;
    }
    
    cur = cur->next;
  }
  return false;
}



bool SocRobXMLNode::getNextAtributeFloat (string name, float& atrib)
{
  xmlChar* key;
  
  if (nodeLibxmlNextSubAtribute_ == NULL) {
    return false;
  }
  
  while ( (nodeLibxmlNextSubAtribute_ = nodeLibxmlNextSubAtribute_->next) != NULL) {
    if ( (!xmlStrcmp (nodeLibxmlNextSubAtribute_->name, (const xmlChar*) name.data()))) {
      key = xmlNodeListGetString (docLibxml_, nodeLibxmlNextSubAtribute_->xmlChildrenNode, 1);
      atrib = atof ( (char*) key);
      xmlFree (key);
      return true;
    }
  }
  return false;
}



bool SocRobXMLNode::getAtributeString (string name, string& atrib)
{
  xmlChar* key = 0;
  xmlNodePtr cur = nodeLibxml_->xmlChildrenNode;
  
  while (cur != NULL) {
    if ( (!xmlStrcmp (cur->name, (const xmlChar*) name.data()))) {
      key = xmlNodeListGetString (docLibxml_, cur->xmlChildrenNode, 1);
      //cout << "DEBUG: catch node " << cur->name <<" = " << key << endl;
      atrib.assign ( (const char*) key);
      xmlFree (key);
      return true;
    }
    
    cur = cur->next;
  }
  
  //if don't find in subnodes, search in properties:
  
  key = xmlGetProp (nodeLibxml_, (const xmlChar*) name.data());
  if (key == 0) {
    return false;
  }
  else {
    atrib.assign ( (const char*) key);
    xmlFree (key);
    return true;
  }
  
  return false;
}



bool SocRobXMLNode::getNextAtributeString (string name, string& atrib)
{
  xmlChar* key;
  
  if (nodeLibxmlNextSubAtribute_ == NULL) {
    return false;
  }
  
  while ( (nodeLibxmlNextSubAtribute_ = nodeLibxmlNextSubAtribute_->next) != NULL) {
    if ( (!xmlStrcmp (nodeLibxmlNextSubAtribute_->name, (const xmlChar*) name.data()))) {
      key = xmlNodeListGetString (docLibxml_, nodeLibxmlNextSubAtribute_->xmlChildrenNode, 1);
      atrib.assign ( (const char*) key);
      xmlFree (key);
      return true;
    }
  }
  return false;
}




string SocRobXMLNode::getName()
{
  return string ( (char*) nodeLibxml_->name);
}
