/**
 * @Filename XmlCharData.cpp
 * @Description XmlCharData implementation file.
 * @Status Implementing
 * @Version $Id: XmlCharData.cpp 5913 2013-02-10 12:15:21Z jreis $
 * @Maintainer Nelson Ramos (nmsra@mega.ist.utl.pt)
 */

#include "XmlCharData.hpp"



using namespace mermaid::support::xml;



/// String containing blank characters.
#define BLANK_STRING (string(" \t\n\r"))



XmlCharData::XmlCharData() :
  string_()
{
}



XmlCharData::XmlCharData (const char* c) :
  string_ (c)
{
  // removeEndBlanks();
}



XmlCharData* XmlCharData::clone()
{
  return new XmlCharData (*this);
}



XmlCharData::XmlCharData (string s) :
  string_ (s)
{
}



XmlCharData::XmlCharData (const XmlCharData& c) :
  string_ (c.string_)
{
}



bool XmlCharData::isBlank() const
{
  size_t pos = string_.find_first_not_of (BLANK_STRING);
  
  if (pos != string::npos) {
    return false;
  }
  else {
    return true;
  }
}



void XmlCharData::removeEndBlanks()
{
  int frontPos = string_.find_first_not_of (BLANK_STRING);
  string_.erase (0, frontPos);
  
  int backPos = string_.find_last_not_of (BLANK_STRING);
  
  string_.erase (backPos + 1, string_.size() - backPos - 1);
}



XmlCharData::operator string() const
{
  return string_;
}



bool XmlCharData::isXmlCharData() const
{
  return true;
}
