/**
 * @Filename XmlEntityLoader.cpp
 * @Description Implementation of the XmlEntityLoader function
 * @Status Implementing
 * @Version $Id: XmlEntityLoader.cpp 5913 2013-02-10 12:15:21Z jreis $
 * @Maintainer Marco Barbosa
 */

#include "XmlEntityLoader.hpp"

#include <string>

#include <libxml/parser.h>

#include <FilePathSearch.hpp>



using namespace mermaid::support::xml;
using std::string;
using mermaid::support::system::FilePathSearch;



xmlExternalEntityLoader XmlEntityLoader::defaultLoader_ = xmlGetExternalEntityLoader();



xmlParserInputPtr XmlEntityLoader::loaderFunction (const char* URL,
    const char* ID,
    xmlParserCtxtPtr context)
{
#if 0
  cerr << "XmlEntityLoader::loaderFunction : ";
  if (URL != 0) {
    cerr << "URL:" << URL;
  }
  if (ID != 0) {
    cerr << " ID:" << ID;
  }
  
  cerr << endl;
#endif
  
  xmlParserInputPtr ret = 0;
  //ret = defaultLoader_ (URL, ID, context);
  
  // default loader could not find file, lets see if its one of the known paths
  
  if (/* (ret == 0) &&*/ (URL != 0)) {
    string fileFullPath;
    
    try {
      // look directly for URL
      fileFullPath = FilePathSearch::getFullPathForFile (string (URL));
    }
    catch (...) {
      // look only for file name
      // cerr << "XmlEntityLoader::loaderFunction : could not find file : " << URL << endl;
      string url (URL);
      int slashPos = url.find_last_of ("/");
      string fileName = url.substr (slashPos + 1);
      // cerr << "XmlEntityLoader::loaderFunction : looking for file : " << fileName << endl;
      
      try {
        fileFullPath = FilePathSearch::getFullPathForFile (fileName);
      }
      catch (...) {
        // cerr << "XmlEntityLoader::loaderFunction : could not find file : " << fileName << endl;
      }
    }
    
    ret = defaultLoader_ (fileFullPath.c_str(), ID, context);
  }
  
  return ret;
}
