#include "mjpeg_camera.h"

#include <cstdio>

#include <ros/ros.h>
#include <cv_bridge/cv_bridge.h>
#include <sensor_msgs/image_encodings.h>
#include <opencv2/highgui/highgui.hpp>



using namespace std;
using namespace ros;



MjpegCamera::
MjpegCamera() :
  camera_nh_ ("camera"),
  it_ (camera_nh_),
  pub_ (it_.advertise ("image_raw", 1, false))
{
}



void
MjpegCamera::
run()
{
  while (ros::ok()) {
    ros::spinOnce();
    
    string filename;
    if (!ros::param::get ("~filename", filename)) {
      ROS_FATAL_STREAM ("Must specify filename to read using parameter ~filename");
      camera_nh_.shutdown();
      abort();
    }
    ROS_INFO_STREAM ("Connecting to: " << filename);
    
    //"curl -Ns <URL of MJPEG Stream>" is executed and the output is read from the FILE pointer stream
    FILE* stream = popen ( ("curl -Ns -m30 " + filename).c_str(), "r");
    int tmp_char = fgetc (stream);
    if ( (!stream) || feof (stream) || ferror (stream) || (EOF == ungetc (tmp_char, stream))) {
      ROS_ERROR_STREAM ("Error opening the stream with curl");
      ros::Duration (1).sleep();
      continue;
    }
    
    //While no error is found
    while (ros::ok() && ! (feof (stream) || ferror (stream))) {
      //No more than 66 chars worth of unused data is read at a time
      char garbage[80];
      
      //Begin Parsing This Frame
      //--ThisRandomString\r\n
      if (! fgets (garbage, sizeof (garbage), stream)) {
        break;
      }
      
      //"Content-type:...\n"
      if (! fgets (garbage, sizeof (garbage), stream)) {
        break;
      }
      
      //"Content-Length: "
      if (16 != fread ( (void*) garbage, sizeof (char), 16, stream)) {
        break;
      }
      
      //Read the frame size
      char framesize_str[10];
      if (! fgets (framesize_str, sizeof (framesize_str), stream)) {
        break;
      }
      std::size_t framesize = atoi (framesize_str);
      ROS_DEBUG_STREAM ("Reading a frame of size " << framesize);
      
      //\n before frame
      if (! fgets (garbage, sizeof (garbage), stream)) {
        break;
      }
      
      //Create space to store the frame
      std::vector<char> data (framesize);
      
      //Read the frame
      if (framesize != fread ( (void*) & (data[0]), sizeof (char), framesize, stream)) {
        break;
      }
      
      //\n before the next boundary
      //fgets(garbage,sizeof(garbage),stream);
      //Finished Parsing This Frame
      
      cv_bridge::CvImage out_msg;
      out_msg.header.stamp = Time::now();
      out_msg.encoding = sensor_msgs::image_encodings::BGR8;
      try {
        out_msg.image = cv::imdecode (cv::Mat (data), 1);
      }
      catch (...) {
        ROS_ERROR_STREAM ("Caught some exception while decoding the received image");
      }
      pub_.publish (out_msg.toImageMsg());
    }
    
    pclose (stream);
  }
}
