/**
 * @Filename Accelerometer.cpp
 * @Description ...to be described...
 * @Status unknown
 * @Version $Id: Accelerometer.cpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis <joaocgreis@gmail.com>
 */

#include "Accelerometer.hpp"
#include <sstream>

#include <String.hpp>
#include <Integer.hpp>
#include <Float.hpp>
#include <Boolean.hpp>

#include <boost/lexical_cast.hpp>

#include <socrob/math.h>
#include "BinaryHelpers.hpp"
#include <MessageDataAcceleration.hpp>



//#include <MessageDataAcceleration.hpp>



using namespace boost;
using namespace std;
using namespace socrob;



namespace
{
  string intStr (int a)
  {
    ostringstream s;
    s << a;
    if (isprint (a)) {
      s << "=='" << static_cast<char> (a) << "'";
    }
    return s.str();
  }
}






Accelerometer::
Accelerometer () :
  FtdiDevice ("accelerometer",
              200000,
              20)
{
  addSerial ("A9007LWD", 115200);
}



void
Accelerometer::
update3()
{
  ROS_INFO_STREAM ("update3 ()");
  
  uint8_t wb[1] = {'r'};
  write (wb, 1);
  
  uint8_t rb[7] = {0};
  read (rb, 7);
  TimeStamp ts;
  
  if (! connected_) {
    ROS_DEBUG_STREAM ("Acceleration read error");
    return;
  }
  
  ROS_DEBUG_STREAM ("Read (" << intStr (rb[0]) << ") (" << intStr (rb[1]) << ") (" << intStr (rb[2]) << ") (" << intStr (rb[3]) << ") (" << intStr (rb[4]) << ") (" << intStr (rb[5]) << ") (" << intStr (rb[6]) << ")");
  
  if (rb[0] != 'r') {
    ROS_INFO ("First byte was not 'r'");
    ftdiError();
    return;
  }
  
  uint16_t xInt;
  uint_cat (xInt, rb[1], rb[2]);
  uint16_t yInt;
  uint_cat (yInt, rb[3], rb[4]);
  uint16_t tInt;
  uint_cat (tInt, rb[5], rb[6]);
  
  // Values are from 0 to 1023 (10 bit ADC), meaning 0 to 5 Volts.
  double xV = static_cast<double> (xInt) * 5 / 1023;
  double yV = static_cast<double> (yInt) * 5 / 1023;
  double tV = static_cast<double> (tInt) * 5 / 1023;
  
  // 0.330 V = 1 G
  double xG = xV / 0.330;
  double yG = yV / 0.330;
  
  // 1 G = 9.78 m/s²
  float x = xG * 9.78;
  float y = yG * 9.78;
  
  // 0.0033 V = 1 degree
  float t = deg2rad (tV / 0.0033);
  
  // Gets 10 consecutive identical values independently on each channel
  // to determine the 0 value
  static double baseX, baseY, baseT;
  {
    static bool stable = false;
    if (! stable) {
      static bool stableX = false, stableY = false, stableT = false;
      if (! stableX) {
        static int strikeX = 0;
        static float lastX = 0;
        if (x == lastX) {
          strikeX++;
          ROS_DEBUG_STREAM ("StrikeX increased to " << strikeX);
          if (10 == strikeX) {
            stableX = true;
            baseX = x;
          }
        }
        else {
          strikeX = 0;
          lastX = x;
        }
      }
      if (! stableY) {
        static int strikeY = 0;
        static float lastY = 0;
        if (y == lastY) {
          strikeY++;
          ROS_DEBUG_STREAM ("StrikeY increased to " << strikeY);
          if (10 == strikeY) {
            stableY = true;
            baseY = y;
          }
        }
        else {
          strikeY = 0;
          lastY = y;
        }
      }
      if (! stableT) {
        static int strikeT = 0;
        static float lastT = 0;
        if (t == lastT) {
          strikeT++;
          ROS_DEBUG_STREAM ("StrikeT increased to " << strikeT);
          if (10 == strikeT) {
            stableT = true;
            baseT = t;
          }
        }
        else {
          strikeT = 0;
          lastT = t;
        }
      }
      if (stableX && stableY && stableT) {
        stable = true;
      }
    }
    if (! stable) {
      return;
    }
  }
  x -= baseX;
  y -= baseY;
  t -= baseT;
  
  ROS_INFO_STREAM ("::: X: " << x << " Y: " << y << " Theta: " << t);
  MessageDataAcceleration::buildAndSend (x, y, t, ts);
}





#define ROS_NAME "accelerometer"
#define ROS_CLASS Accelerometer
#include <RosMain.cpp.part>
