#include "ArduinoConverter.hpp"
#include <sstream>

#include <opencv2/core/core.hpp>

#include <String.hpp>
#include <Integer.hpp>
#include <Float.hpp>
#include <Boolean.hpp>
#include <TrackedPosition3D.hpp>

#include <boost/lexical_cast.hpp>

#include <socrob/math.h>
using namespace socrob;

#include <cfloat>

#include "BinaryHelpers.hpp"

#include <socrob_ros/CompassAngle.h>
#include <MessageDataCompass.hpp>



ArduinoConverter::
ArduinoConverter () :
  nh_(),
  angle_pub_ (nh_.advertise<socrob_ros::CompassAngle> ("compass_angle", 1)),
  raw_sub_ (nh_.subscribe ("compass_raw", 1, &ArduinoConverter::rawCallback, this))
{
}



void
ArduinoConverter::
rawCallback (const std_msgs::UInt16::ConstPtr& msg)
{
  float angleFloat = 0;
  
  // deg2rad and div by 10. Be careful with the sign (might change in the southern hemisphere)
  {
    bool change_sign;
    if (! ros::param::getCached ("~change_sign", change_sign)) {
      change_sign = false;
      ros::param::set ("~change_sign", change_sign);
    }
    
    if (change_sign) {
      angleFloat = -clipAngle (deg2rad (msg->data / 10.0));
    }
    else {
      angleFloat = clipAngle (deg2rad (msg->data / 10.0));
    }
  }
  
  // ArduinoConverter returns -0 sometimes.
  // This is the correct way of checkig if a floating point number is not 0 (!=0 is not correct)
  if (abs (angleFloat) > FLT_EPSILON) {
    socrob_ros::CompassAngle msg;
    msg.header.stamp = ros::Time::now();
    msg.angle = angleFloat;
    angle_pub_.publish (msg);
  }
  else {
    ROS_INFO_STREAM ("Attention: Received angle " << angleFloat << ", ignoring.");
  }
}



int main (int argc, char** argv)
{
  ros::init (argc, argv, "arduino_converter");
  ArduinoConverter ac;
  ros::spin();
  return 0;
}
