/**
* @Filename BinaryHelpers.hpp
* @Description Helper functions to manage binary information.
* @Status Implementing
* @Version $Id: BinaryHelpers.hpp 5393 2012-02-13 22:31:50Z jreis $
* @Maintainer Joao Reis <joaocgreis@gmail.com>
*/

#ifndef __BINARY_HELPERS_HPP__
#define __BINARY_HELPERS_HPP__

#include <stdint.h>



/**
 * Concatenates two 8 bit integers.
 * @Argument result - the concatenated 16 bit integer
 * @Argument left - the first 8 bits
 * @Argument right - the last 8 bits
 * @Author Joao Reis <joaocgreis@gmail.com>
 */
inline
void
uint_cat (uint16_t& result,
          const uint8_t& left,
          const uint8_t& right)
{
  result = left;
  result <<= 8;
  result += right;
}



/**
 * Splits one 16 bit integer to two 8 bit integers.
 * @Argument left - the first 8 bits
 * @Argument right - the last 8 bits
 * @Argument source - the 16 bit integer to be split
 * @Author Joao Reis <joaocgreis@gmail.com>
 */
inline
void
uint_split (uint8_t& left,
            uint8_t& right,
            const uint16_t& source)
{
  left = source >> 8;
  right = source & 0xFF;
}

#endif
