/**
 * @Filename Compass.cpp
 * @Description ...to be described...
 * @Status unknown
 * @Version $Id: Compass.cpp 6116 2013-06-29 14:11:55Z jreis $
 * @Maintainer Joao Reis <joaocgreis@gmail.com>
 */

#include "Compass.hpp"
#include <sstream>

#include <opencv2/core/core.hpp>

#include <String.hpp>
#include <Integer.hpp>
#include <Float.hpp>
#include <Boolean.hpp>
#include <TrackedPosition3D.hpp>

#include <boost/lexical_cast.hpp>

#include <socrob/math.h>
using namespace socrob;

#include <cfloat>

#include "BinaryHelpers.hpp"

#include <MessageDataCompass.hpp>

#include <socrob_ros/TemperatureStamped.h>



// HACK FOR NOW, AS ALWAYS
#include <fstream>




using namespace boost;
using namespace std;



#undef WRITE_OUT_COMPASS_DATA
//#define WRITE_OUT_COMPASS_DATA



#undef USE_TEMPERATURE_SENSORS
#define USE_TEMPERATURE_SENSORS



#undef USE_GRABBER
//#define USE_GRABBER
#undef CONTROL_GRABBER_BY_CONTROL_INPUT_FINAL
//#define CONTROL_GRABBER_BY_CONTROL_INPUT_FINAL



namespace
{
  string intStr (int a)
  {
    ostringstream s;
    s << a;
    if (isprint (a)) {
      s << "=='" << static_cast<char> (a) << "'";
    }
    return s.str();
  }
}



Compass::
Compass () :
  FtdiDevice ("compass",
              200000,
              20),
  nh_(),
  private_nh_ ("~"),
  calibration_service_ (private_nh_.advertiseService ("calibrate", &Compass::calibrate, this)),
  temperature_pub_ (nh_.advertise<socrob_ros::TemperatureStamped> ("temperature", 1, true)),
  kick_sub_ (nh_.subscribe ("kick", 1, &Compass::kickCallback, this)),
#ifdef CONTROL_GRABBER_BY_CONTROL_INPUT_FINAL
  motors_sub_ (nh_.subscribe ("control_input_final", 1, &Compass::controlFinalCallback, this)),
#else
  velocity_sub_ (nh_.subscribe ("sensor_fusion/velocity", 1, &Compass::velocityCallback, this)),
#endif
  last_close_ball_time_ (ros::TIME_MIN),
  last_has_ball_time_ (ros::TIME_MIN),
  last_kick_ (ros::TIME_MIN),
  last_control_ (new socrob_ros::ControlInput()),
  debug_ ("debug"),
  last_temperature_request_ (ros::TIME_MIN)
{
  std::string serial;
  if (ros::param::get ("~serial", serial)) {
    addSerial (serial, 115200);
  }
  else {
    addSerial ("A800ewuq", 115200); //
    addSerial ("A9007LWD", 115200); // OMNI1
    addSerial ("A9007M5z", 115200); // OMNI2
    addSerial ("A800evl5", 115200); // OMNI3
    addSerial ("A800evfT", 115200); // OMNI4
  }
}



void
Compass::
kickCallback (const std_msgs::Float32::ConstPtr& msg)
{
  last_kick_ = ros::Time::now();
  
  //   uint8_t wb[3] = {'v'};
  //   uint_split (wb[1], wb[2], 511);
  //   write (wb, 3);
}



void
Compass::
controlFinalCallback (const socrob_ros::ControlInput::ConstPtr& msg)
{
  last_control_ = msg;
}



void
Compass::
velocityCallback (const socrob_ros::VelocityPostureStamped::ConstPtr& msg)
{
  last_velocity_ = msg;
}



void
Compass::
update3()
{
  ROS_DEBUG_STREAM ("update3 ()");
  
  ros::Time time_now = ros::Time::now();
  
  float angleFloat = 0;
  int temp1 = 0, temp2 = 0, temp3 = 0;
  
  {
    uint8_t wb[1] = {'g'};
    write (wb, 1);
    
    uint8_t rb[3] = {0};
    read (rb, 3);
    TimeStamp ts;
    
    if (! connected_) {
      ROS_ERROR_STREAM ("Angle read error");
      return;
    }
    
    ROS_DEBUG_STREAM ("Read (" << intStr (rb[0]) << ") (" << intStr (rb[1]) << ") (" << intStr (rb[2]) << ")");
    
    if (rb[0] != 'g') {
      ROS_ERROR_STREAM ("First byte was not 'g'");
      ftdiError();
      return;
    }
    
    uint16_t angle;
    uint_cat (angle, rb[1], rb[2]);
    
    // deg2rad and div by 10. Be careful with the sign (might change in the southern hemisphere)
    {
      bool change_sign;
      if (! ros::param::getCached ("~change_sign", change_sign)) {
        change_sign = false;
        ros::param::set ("~change_sign", change_sign);
      }
      
      if (change_sign) {
        angleFloat = -clipAngle (deg2rad (angle / 10.0));
      }
      else {
        angleFloat = clipAngle (deg2rad (angle / 10.0));
      }
    }
    
    // Compass returns -0 sometimes.
    // This is the correct way of checkig if a floating point number is not 0 (!=0 is not correct)
    if (abs (angleFloat) > FLT_EPSILON) {
      MessageDataCompass::buildAndSend (angleFloat, ts);
    }
    else {
      ROS_INFO_STREAM ("Attention: Received angle " << angleFloat << ", ignoring.");
    }
    
#ifdef WRITE_OUT_COMPASS_DATA
    static ofstream output ("/opt/socrob/log/compassData");
    output << angleFloat << " " << (long) ts.getTimeInMs() << endl;
#endif
  }
  
  
#ifdef USE_GRABBER
  // 0 -> reverse max
  // 255 -> stopped
  // 511 -> max
  uint16_t v = 255;
  
  shared_ptr<TrackedPosition3D const> ball = dynamic_pointer_cast<TrackedPosition3D const> (getDataWI ("BallFullPosition"));
  if ( (ball->seeTrackedObject == true)
       && (ball->localPosition.getDistanceInXYPlane() < 2)) {
    last_close_ball_time_ = time_now;
  }
  if ( (ball->seeTrackedObject == true)
       && (ball->localPosition.getDistanceInXYPlane() < 0.35)
       && (abs (ball->localPosition.getAzimuth()) < (M_PI / 12))) {
    last_has_ball_time_ = time_now;
  }
  
  if ( (time_now - last_kick_) < ros::Duration (1)) {
    // Kick with grabber.
    v = 511;
  }
  else if ( (! ball->seeTrackedObject)
            || ( (time_now - last_has_ball_time_) < ros::Duration (3))) {
    // Has ball. Tight control depending on motors.
#ifdef CONTROL_GRABBER_BY_CONTROL_INPUT_FINAL
    if (!last_control_) {
      ROS_WARN_STREAM_THROTTLE (1, "No velocity info yet, cannot control grabber.");
      return;
    }
    if (last_control_->type != socrob_ros::ControlInput::VELOCITY) {
      ROS_ERROR_STREAM_THROTTLE (1, "Grabber only supports velocity control");
      return;
    }
    double vX = last_control_->linear.vX;
#else
    if (!last_velocity_) {
      ROS_WARN_STREAM_THROTTLE (1, "No velocity info yet, cannot control grabber.");
      return;
    }
    double vX = last_velocity_->velocityPosture.vel2D.vX;
#endif
    
    double vv;
    if ( (last_control_->linear.vX) <= 0) {
      vv = 255 + 0.3 * 255 * vX - 50;
    }
    else {
      vv = 255 + 0.3 * 255 * vX - 10;
    }
    if (vv < 100) {
      v = 100;
    }
    else if (vv > 411) {
      v = 411;
    }
    else {
      v = vv;
    }
  }
  else if ( (time_now - last_close_ball_time_) < ros::Duration (3)) {
    // Ball is close. Try to acquire.
    v = 100;
  }
  else {
    // Ball is far away.
    v = 255;
  }
  
  uint8_t wb[3] = {'v'};
  uint_split (wb[2], wb[1], v);
  write (wb, 3);
  
  ROS_DEBUG_STREAM ("Wrote (" << intStr (wb[0]) << ") (" << intStr (wb[1]) << ") (" << intStr (wb[2]) << ")  --> " << v);
  
  uint8_t rb[1] = {0};
  read (rb, 1);
  if (! connected_) {
    ROS_ERROR_STREAM ("Grabber write error");
    return;
  }
  
  if (rb[0] != 'v') {
    ROS_ERROR_STREAM ("First byte was not 't'");
    ftdiError();
    return;
  }
#endif
  
  
#ifdef USE_TEMPERATURE_SENSORS
  //   if ( (last_temperature_request_ + ros::Duration (30, 0)) < ros::Time::now()) {
  //     last_temperature_request_ = ros::Time::now();
  {
    uint8_t wb[1] = {'t'};
    write (wb, 1);
    
    uint8_t rb[4] = {0};
    read (rb, 4);
    if (! connected_) {
      ROS_ERROR_STREAM ("Temperature read error");
      return;
    }
    
    ROS_DEBUG_STREAM ("Read (" << intStr (rb[0]) << ") (" << intStr (rb[1]) << ") (" << intStr (rb[2]) << ") (" << intStr (rb[3]) << ")");
    
    if (rb[0] != 't') {
      ROS_ERROR_STREAM ("First byte was not 't'");
      ftdiError();
      return;
    }
    
    temp1 = static_cast<int> (rb[1]);
    temp2 = static_cast<int> (rb[2]);
    temp3 = static_cast<int> (rb[3]);
    
    // sensors only go up to 125 i think
    if (temp1 > 150) {
      temp1 = 0;
    }
    if (temp2 > 150) {
      temp2 = 0;
    }
    if (temp3 > 150) {
      temp3 = 0;
    }
    
    if ( (temp1 > 70) || (temp2 > 70) || (temp3 > 70)) {
      ROS_FATAL_STREAM ("TEMPERATURE TOO HIGH!!! Sensor 1: " << temp1 << " degrees; Sensor 2: " << temp2 << " degrees; Sensor 3: " << temp3 << " degrees");
    }
    
    // Used in Istanbul. Mini terminal always visible above the interface, ssh to the laptop, tail -f /tmp/temperaturelog
    {
      static ofstream fs ("/tmp/temperaturelog");
      fs << endl << temp1 << " " << temp2 << " " << temp3;
      fs.flush();
    }
    
    {
      socrob_ros::TemperatureStamped temp;
      temp.header.stamp = ros::Time::now();
      temp.temp1 = temp1;
      temp.temp2 = temp2;
      temp.temp3 = temp3;
      temperature_pub_.publish (temp);
    }
  }
#endif
  
  if (debug_.active()) {
    cv::Mat image (480, 640, CV_8UC3, cv::Scalar (255, 255, 255));
    
    cv::circle (image,
                cv::Point (240, 240),
                220,
                cv::Scalar (0, 0, 0),
                5);
                
    cv::line (image,
              cv::Point (240, 240),
              cv::Point (240 + 220 * cos (-angleFloat), 240 + 220 * sin (-angleFloat)),
              cv::Scalar (0, 0, 0),
              3);
              
    cv::putText (image,
                 lexical_cast<string> (temp1),
                 cv::Point (500, 200),
                 cv::FONT_HERSHEY_SIMPLEX,
                 2.8,
                 cv::Scalar (0, 0, 0));
                 
    cv::putText (image,
                 lexical_cast<string> (temp2),
                 cv::Point (500, 300),
                 cv::FONT_HERSHEY_SIMPLEX,
                 2.8,
                 cv::Scalar (0, 0, 0));
                 
    cv::putText (image,
                 lexical_cast<string> (temp3),
                 cv::Point (500, 400),
                 cv::FONT_HERSHEY_SIMPLEX,
                 2.8,
                 cv::Scalar (0, 0, 0));
                 
    debug_.set (image);
  }
  
  ROS_INFO_STREAM_THROTTLE (10, "T1: " << temp1 << "  T2: " << temp2 << "  T3: " << temp3 << "  A: " << angleFloat);
  ROS_DEBUG_STREAM_THROTTLE (1, "T1: " << temp1 << "  T2: " << temp2 << "  T3: " << temp3 << "  A: " << angleFloat);
}



bool
Compass::
calibrate (std_srvs::Empty::Request&,
           std_srvs::Empty::Response&)
{
  ROS_INFO_STREAM ("Calibrating compass...");
  
  uint8_t wb[1] = {'c'};
  write (wb, 1);
  
  return true;
}





#define ROS_NAME "compass"
#define ROS_CLASS Compass
#include <RosMain.cpp.part>
