/**
 * @Filename Compass.hpp
 * @Description ...to be described...
 * @Status unknown
 * @Version $Id: Compass.hpp 6113 2013-06-29 12:27:35Z jreis $
 * @Maintainer Joao Reis <joaocgreis@gmail.com>
 */

#ifndef _COMPASS_HPP_
#define _COMPASS_HPP_

#include <std_msgs/Float32.h>
#include <socrob_ros/ControlInput.h>
#include <socrob_ros/VelocityPostureStamped.h>
#include <std_srvs/Empty.h>

#include <socrob/display.h>

#include "FtdiDevice.hpp"



/**
 * @Class Compass Compass.hpp "Compass.hpp"
 * @Author Joao Reis
 */
class Compass :
  public FtdiDevice
{
  public:
    Compass ();
    
    bool
    calibrate (std_srvs::Empty::Request& request,
               std_srvs::Empty::Response& response);
               
  private:
    void
    kickCallback (const std_msgs::Float32::ConstPtr& msg);
    
    void
    controlFinalCallback (const socrob_ros::ControlInput::ConstPtr& msg);
    
    void
    velocityCallback (const socrob_ros::VelocityPostureStamped::ConstPtr& msg);
    
    void update3();
    
    ros::NodeHandle nh_;
    ros::NodeHandle private_nh_;
    
    ros::ServiceServer calibration_service_;
    ros::Publisher temperature_pub_;
    ros::Subscriber kick_sub_;
    ros::Subscriber motors_sub_;
    ros::Subscriber velocity_sub_;
    
    ros::Time last_close_ball_time_;
    ros::Time last_has_ball_time_;
    ros::Time last_kick_;
    socrob_ros::ControlInput::ConstPtr last_control_;
    socrob_ros::VelocityPostureStamped::ConstPtr last_velocity_;
    
    socrob::Display debug_;
    
    ros::Time last_temperature_request_;
};

#endif
