/**
 * @Filename FtdiDevice.hpp
 * @Description Helper to use FTDI devices
 * @Status unknown
 * @Version $Id: FtdiDevice.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis (joaocgreis@gmail.com)
 */

#ifndef _FTDI_DEVICE_HPP_
#define _FTDI_DEVICE_HPP_



class FtdiDevice;

#include <stdint.h>
#include <ftdi.hpp>

#include <SocRobService.hpp>



/**
 * @Class FtdiDevice FtdiDevice.hpp "FtdiDevice.hpp"
 * @Author Joao Reis
 */
class FtdiDevice :
  public SocRobService
{
  protected:
    FtdiDevice (string const& deviceName,
                long const usecTimeout,
                float const rate);
                
    virtual void afterConnect() {}
    virtual void update3() {};
    
    void addSerial (string const& serial, int baudrate);
    
    void write (uint8_t* const buf, unsigned const count);
    void read (uint8_t* const buf, unsigned const count);
    
    void ftdiError();
    
  protected:
    float rate_;
    bool connected_;
    
  private:
    bool initialize();
    bool connectDevice();
    void update();
    
    long timeout_;
    map<string, int> serials_;
    Ftdi::Context ctx_;
    
    string name_;
};

#endif
