/**
 * @Filename Kicker.cpp
 * @Description ...to be described...
 * @Status unknown
 * @Version $Id: Kicker.cpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis <joaocgreis@gmail.com>
 */

#include "Kicker.hpp"

#include <sstream>

#include <String.hpp>
#include <Integer.hpp>
#include <Boolean.hpp>

#include <MessageEvent.hpp>

#include <boost/lexical_cast.hpp>



#define KICKERON_THRESHOLD (45.0)
#include <boost/concept_check.hpp>



using namespace boost;
using namespace std;



namespace
{
#define RED    "\033[1;31m"
#define GREEN  "\033[1;32m"
#define YELLOW "\033[1;33m"
#define NORMAL "\033[0m"
#define BLUE   "\033[34;1m"



  // shamelessly copied from omniManual
  void noisyPrint (string s)
  {
    printf ("\n\r"
            "\n\r"
            "-------------------------------------------------------\n\r"
            YELLOW
            "%s\n\r"
            NORMAL
            "-------------------------------------------------------\n\r"
            "\n\r", s.c_str());
    fflush (stdout);
  }
}



namespace
{
  string intStr (int a)
  {
    ostringstream s;
    s << a;
    if (isprint (a)) {
      s << "=='" << static_cast<char> (a) << "'";
    }
    return s.str();
  }
}






Kicker::
Kicker () :
  FtdiDevice ("kicker",
              200000,
              4),
  shouldBeCharging_ (false)
{
  addSerial ("A800flpX", 115200);
  addSerial ("A800flpY", 115200); // OMNI2
  addSerial ("A8008z9j", 115200); // OMNI3
  
  putDataWI ("KickerOn", shared_ptr<Boolean> (new Boolean (false)));
}



void
Kicker::
kick (int strength)
{
  ROS_INFO_STREAM ("kick (strength = " + lexical_cast<string> (strength) + ")");
  
  kickPrivate (strength);
  
  MessageEvent::buildAndSend ("KICKED");
}



void
Kicker::
update3()
{
  ROS_INFO_STREAM ("update3 ()");
  
  bool isRunning;
  {
    shared_ptr<Boolean const> isRunning_wi = dynamic_pointer_cast<Boolean const> (getDataWI ("isRunning"));
    isRunning = isRunning_wi ? isRunning_wi->boolValue() : false;
  }
  
  if (isRunning != shouldBeCharging_) {
    // Send anything only if the button changed
    ROS_INFO_STREAM ("Detected button change");
    if (isRunning) {
      startKickerPrivate();
    }
    else {
      stopKickerPrivate();
      // Discharge provokes timeouts... Let the HW manage discharging.
      // dischargeKickerPrivate();
    }
  }
  
  getVoltagePrivate();
}



void
Kicker::
afterConnect()
{
  ROS_INFO_STREAM ("afterConnect ()");
  
  bool isRunning;
  {
    shared_ptr<Boolean const> isRunning_wi = dynamic_pointer_cast<Boolean const> (getDataWI ("isRunning"));
    isRunning = isRunning_wi ? isRunning_wi->boolValue() : false;
  }
  
  if (isRunning) {
    startKickerPrivate();
  }
  else {
    stopKickerPrivate();
    // Dischage takes a lot of time, provoking timeouts.
    // If we discharge right after connect, we are always disconecting,
    // connecting and discharging in a loop.
    // dischargeKickerPrivate();
  }
}



void
Kicker::
kickPrivate (int strength)
{
  const char KICK_COMMAND = 'k';
  
  ROS_INFO_STREAM ("kickPrivate (strength = " + lexical_cast<string> (strength) + ")");
  if (! connected_) {
    ROS_DEBUG_STREAM ("Can only be used when connected to the kicker... returning.");
    return;
  }
  
  uint8_t wb[2];
  
  wb[0] = KICK_COMMAND;
  
  if (strength <= 25) {
    if (strength < 0) {
      ROS_ERROR_STREAM ("Received kick command with strength < 0");
    }
    wb[1] = 1;
  }
  else if (strength <= 50) {
    wb[1] = 2;
  }
  else if (strength <= 75) {
    wb[1] = 3;
  }
  else {
    if (strength > 100) {
      ROS_ERROR_STREAM ("Received kick command with strength > 100");
    }
    wb[1] = 4;
  }
  
  ROS_DEBUG_STREAM ("Going to write '" << KICK_COMMAND << "' '"
                    << static_cast<int> (wb[1]) << "'");
  write (wb, 2);
  if (! connected_) {
    ROS_DEBUG_STREAM ("Write error");
    updateKickerOn (false);
    return;
  }
  
  ROS_INFO_STREAM ("KICK! Kicked with strength " << strength);
  
  // To update KickerOn
  getVoltagePrivate();
}



float
Kicker::
getVoltagePrivate()
{
  const char GET_VOLTAGE_COMMAND = 'v';
  
  ROS_INFO_STREAM ("getVoltagePrivate ()");
  if (! connected_) {
    ROS_DEBUG_STREAM ("Can only be used when connected to the kicker... returning.");
    return 0;
  }
  
  uint8_t wb[1];
  wb[0] = GET_VOLTAGE_COMMAND;
  write (wb, 1);
  if (! connected_) {
    ROS_DEBUG_STREAM ("Write error");
    updateKickerOn (false);
    return 0;
  }
  ROS_DEBUG_STREAM ("Written '" << GET_VOLTAGE_COMMAND << "'");
  
  uint8_t rb[1] = {0};
  read (rb, 1);
  if (! connected_) {
    ROS_DEBUG_STREAM ("Read error");
    updateKickerOn (false);
    return 0;
  }
  ROS_DEBUG_STREAM ("Read " << intStr (rb[0]));
  
  float V = static_cast<float> (rb[0]) / 2;
  
  ROS_INFO_STREAM ("Kicker is at " << V << "V.");
  
  updateKickerOn (V > KICKERON_THRESHOLD);
  
  return V;
}



void
Kicker::
startKickerPrivate()
{
  const char START_COMMAND = 'c';
  const char START_ANSWER = 'C';
  
  ROS_INFO_STREAM ("startKickerPrivate ()");
  if (! connected_) {
    ROS_DEBUG_STREAM ("Can only be used when connected to the kicker... returning.");
    return;
  }
  
  uint8_t wb[1];
  wb[0] = START_COMMAND;
  write (wb, 1);
  if (! connected_) {
    ROS_DEBUG_STREAM ("Write error");
    updateKickerOn (false);
    return;
  }
  ROS_DEBUG_STREAM ("Written '" << START_COMMAND << "'");
  
  uint8_t rb[1] = {0};
  read (rb, 1);
  if (! connected_) {
    ROS_DEBUG_STREAM ("Read error");
    updateKickerOn (false);
    return;
  }
  ROS_DEBUG_STREAM ("Read " << intStr (rb[0]));
  
  if (rb[0] != START_ANSWER) {
    ROS_ERROR_STREAM ("Start kicker answer was " << intStr (rb[0])
                      << ". Expected " << intStr (START_ANSWER) << ". Disconnected.");
    updateKickerOn (false);
    ftdiError();
    return;
  }
  
  shouldBeCharging_ = true;
  
  // To update KickerOn
  getVoltagePrivate();
  
  ROS_INFO_STREAM ("Started the kicker.");
}



void
Kicker::
stopKickerPrivate()
{
  const char STOP_COMMAND = 'b';
  const char STOP_ANSWER = 'B';
  
  ROS_INFO_STREAM ("stopKickerPrivate ()");
  if (! connected_) {
    ROS_DEBUG_STREAM ("Can only be used when connected to the kicker... returning.");
    return;
  }
  
  uint8_t wb[1];
  wb[0] = STOP_COMMAND;
  write (wb, 1);
  if (! connected_) {
    ROS_DEBUG_STREAM ("Write error");
    updateKickerOn (false);
    return;
  }
  ROS_DEBUG_STREAM ("Written '" << STOP_COMMAND << "'");
  
  uint8_t rb[1] = {0};
  read (rb, 1);
  if (! connected_) {
    ROS_DEBUG_STREAM ("Read error");
    updateKickerOn (false);
    return;
  }
  ROS_DEBUG_STREAM ("Read " << intStr (rb[0]));
  
  if (rb[0] != STOP_ANSWER) {
    ROS_ERROR_STREAM ("Stop kicker answer was " << intStr (rb[0])
                      << ". Expected " << intStr (STOP_ANSWER) << ". Disconnected.");
    updateKickerOn (false);
    ftdiError();
    return;
  }
  
  shouldBeCharging_ = false;
  
  updateKickerOn (false);
  
  ROS_INFO_STREAM ("Stopped the kicker.");
}



void
Kicker::
dischargeKickerPrivate()
{
  const char DISCHARGE_COMMAND = 'd';
  const char DISCHARGE_ANSWER = 'D';
  
  ROS_INFO_STREAM ("dischargeKickerPrivate ()");
  if (! connected_) {
    ROS_DEBUG_STREAM ("Can only be used when connected to the kicker... returning.");
    return;
  }
  
  uint8_t wb[1];
  wb[0] = DISCHARGE_COMMAND;
  write (wb, 1);
  if (! connected_) {
    ROS_DEBUG_STREAM ("Write error");
    updateKickerOn (false);
    return;
  }
  ROS_DEBUG_STREAM ("Written '" << DISCHARGE_COMMAND << "'");
  
  uint8_t rb[1] = {0};
  read (rb, 1);
  if (! connected_) {
    ROS_DEBUG_STREAM ("Read error");
    updateKickerOn (false);
    return;
  }
  ROS_DEBUG_STREAM ("Read " << intStr (rb[0]));
  
  if (rb[0] != DISCHARGE_ANSWER) {
    ROS_ERROR_STREAM ("Discharge kicker answer was " << intStr (rb[0])
                      << ". Expected " << intStr (DISCHARGE_ANSWER) << ". Disconnected.");
    updateKickerOn (false);
    ftdiError();
    return;
  }
  
  updateKickerOn (false);
  
  ROS_INFO_STREAM ("Discharged the kicker.");
}



void
Kicker::
updateKickerOn (bool kickerOn)
{
  if (kickerOn) {
    ROS_DEBUG_STREAM ("Enabling KickerOn in WI");
  }
  else {
    ROS_DEBUG_STREAM ("Disabling KickerOn in WI");
  }
  
  putDataWI ("KickerOn", shared_ptr<Boolean> (new Boolean (kickerOn)));
}





#define ROS_NAME "kicker"
#define ROS_CLASS Kicker
#include <RosMain.cpp.part>
