#include "control_selector.h"

#include <boost/bind.hpp>



using namespace std;
using namespace ros;



ControlSelector::
ControlSelector () :
  last_published_ (0),
  nh_(),
  final_pub_ (nh_.advertise<socrob_ros::ControlInput> ("control_input_final", 1, true))
{
  /// Add topics to subscribe here, by increasing order of priority
  add ("control_input_navigation");
  add ("control_input_manual");
  add ("control_input");
}



void
ControlSelector::
add (string const& topic)
{
  size_t number = subs_.size();
  boost::function<void (socrob_ros::ControlInput::ConstPtr const&) > callback = boost::bind (&ControlSelector::callback, this, number, _1);
  subs_.push_back (nh_.subscribe (topic, 1, callback));
}



void
ControlSelector::
callback (size_t number,
          socrob_ros::ControlInput::ConstPtr const& msg)
{
  if (last_published_ > number) {
    for (size_t i = number + 1; i < subs_.size(); ++i) {
      if (subs_[i].getNumPublishers() > 0) {
        return;
      }
    }
  }
  
  final_pub_.publish (msg);
  
  if (last_published_ != number) {
    ROS_INFO_STREAM ("Changing control input source to: " << subs_[number].getTopic());
    last_published_ = number;
  }
}
