#ifndef _DEVICES_CONSTS_HPP_
#define _DEVICES_CONSTS_HPP_

#include <string>



namespace constants
{
  namespace AllWebotsDevices
  {
    /// These are settings to add realism to the Webots simulations,
    /// which include sensors ranges, errors and failures
    
    /// Desired standard deviation for the results of the self-localization algorithms (meters or rad)
    /// The noise present in self-localization is assumed to be additive Gaussian
    const float self_localization_noise_x = 0;
    const float self_localization_noise_y = 0;
    const float self_localization_noise_theta = 0;
    
    /// Constant offset (disturbance) to be added to the self-localization of the robot,
    /// to simulate effects such as damaged or miscalibrated cameras (meters and radians)
    const float self_localization_offset_x = 0;
    const float self_localization_offset_y = 0;
    const float self_localization_offset_theta = 0;
    
    /// Expected time before a self-localization failure happens and the robot becomes lost (seconds).
    /// Sampled from an exponential distribution. Use -1 to turn off
    const float self_localization_failure = -1;
    /// Expected time for the robot to become localized again after it becomes lost (seconds).
    /// Sampled from an exponential distribution. Use -1 to turn off
    const float self_localization_recovery = -1;
    
    ///Standard deviation for the noise on the robot's odometry (meters or rad)
    /// Additive Gaussian noise
    const float odometry_noise_x = 0;
    const float odometry_noise_y = 0;
    const float odometry_noise_theta = 0;
    
    /// Probability of not detecting the ball in each iteration, even if it is inside the detection range
    const float ball_localization_failure_rate = 0;
    /// Desired standard deviation for the results of the ball-localization algorithms (meters)
    /// The noise present in ball-localization is assumed to be multiplicative (linear), i.e.,
    /// the further away the ball is, the greater the noise that it will be under.
    /// Calibrate these values as if the noise was Gaussian for a given distance, and then
    /// select the gain accordingly
    const float ball_localization_noise_x = 0;
    const float ball_localization_noise_y = 0;
    /// Linear gain for the noise. Use -1 to make the noise additive
    const float ball_localization_noise_gain = 1;
    
    /// Desired standard deviation for in the detection of the velocity of the ball (meters/seconds)
    /// As in the case of ball localization, this is multiplicative noise by default
    const float ball_velocity_noise_x = 0;
    const float ball_velocity_noise_y = 0;
    /// Use -1 to make the noise additive
    const float ball_velocity_noise_gain = 1;
    
    /// Range at which the robot is able to detect obstacles (meters)
    const float obstacles_detection_range = 6;
    /// Probability of not detecting an obstacle inside the detection range (for one iteration)
    const float obstacles_detection_failure_rate = 0;
    
    /// Desired standard deviation for obstacle-detection purposes (meters or rad). Multiplicative noise
    const float obstacles_noise_distance = 0;
    const float obstacles_noise_angle = 0;
    /// Use -1 to make the noise additive
    const float obstacles_noise_gain = 1;
    
    /// Expected number of times the kickers can be used without breaking. Use -1 to turn off
    const float kicker_expected_uses = -1;
  }
  
  namespace OfflineDataStream
  {
    const std::string datapathname = "offlineData/";
    const std::string lutname = "lut6410hOMNI1.so";
    const int ImageWidth = 640;
    const int ImageHeight = 480;
  }
}

#endif
