#ifndef LINE_POINTS_H
#define LINE_POINTS_H

#include <ros/ros.h>
#include <image_transport/image_transport.h>

#include <SocRobService.hpp>

#include "search_cache.h"
#include "transform_cache.h"

#include <socrob/display.h>



class LinePoints:
  public SocRobService
{
  public:
    LinePoints ();
    
    bool initialize (void);
    
  private:
    const int W;
    const int H;
    
    void
    imageCallback (const sensor_msgs::ImageConstPtr& image);
    
    void
    computePointsToSearch ();
    
    void
    detectTransitions (vector<pair<cv::Point, cv::Point> >& transitions,
                       vector<cv::Point>& obstacles,
                       cv::Mat const& gray);
                       
    shared_ptr<set<pair<float, float> > const>
    toRobotFrame_set (vector<pair<cv::Point, cv::Point> > const& transitions);
    shared_ptr<vector<pair<float, float> > const>
    toRobotFrame (vector<pair<cv::Point, cv::Point> > const& transitions);
    shared_ptr<vector<pair<float, float> > const>
    toRobotFrame (vector<cv::Point> const& obstaclePoints);
    
    vector<cv::Point>
    toImageFrame (shared_ptr<vector<pair<float, float> > const> transitions);
    
    uint8_t highestBlackValue_;
    
    SearchCache searchPoints_;
    TransformCache cameraTransform_;
    
    ros::NodeHandle nh_;
    image_transport::ImageTransport it_;
    
    socrob::Display debug_;
    socrob::Display debug_mask_;
    
    image_transport::Subscriber image_sub_;
};

#endif
