#include "search_cache.h"

#include "sensors_consts.hpp"

#include <set>

#include <WorldDefinition.hpp>
#include <WorldInfo.hpp>
#include <Integer.hpp>
#include <socrob/math.h>
using namespace socrob;



SearchCache::
SearchCache (int W,
             int H)
{
  unsigned radiusOfSearch = WD::SearchableRadius() + constants::LinePoints::radius_increment;
  
  const cv::Point ORIGIN (WD::CameraCenterImgCol(), WD::CameraCenterImgRow());
  const cv::Point ROBOT_CENTER (WD::RobotCenterImgCol(), WD::RobotCenterImgRow());
  
  const double SUPPORT_1_MIN = WD::Support1Min();
  const double SUPPORT_1_MAX = WD::Support1Max();
  const double SUPPORT_2_MIN = WD::Support2Min();
  const double SUPPORT_2_MAX = WD::Support2Max();
  const double SUPPORT_3_MIN = WD::Support3Min();
  const double SUPPORT_3_MAX = WD::Support3Max();
  
  set<pair<int, int> > used_points;
  points = vector<vector<cv::Point> > (constants::LinePoints::num_radial_rays_to_search, vector<cv::Point>());
  
  for (size_t j = 0; j < constants::LinePoints::num_radial_rays_to_search; j++) {
    double angle = j * 2 * M_PI / constants::LinePoints::num_radial_rays_to_search;
    
    if (isAngleBetween (angle, SUPPORT_1_MIN, SUPPORT_1_MAX)
        || isAngleBetween (angle, SUPPORT_2_MIN, SUPPORT_2_MAX)
        || isAngleBetween (angle, SUPPORT_3_MIN, SUPPORT_3_MAX)) {
      continue;
    }
    
    for (unsigned i = (j % 2 == 0) ? 0 : constants::LinePoints::radius_where_the_alternate_rows_start;
         i < radiusOfSearch;
         i += 1 + constants::LinePoints::pixel_jump_in_each_row) {
      cv::Point point = ORIGIN;
      point.x += floor (i * cos (angle));
      point.y += floor (i * -sin (angle));
      if (! (isBetween (point.x, 0, W) && isBetween (point.y, 0, H))) {
        continue;
      }
      
      bool in_robot = (sqr (ROBOT_CENTER.x - point.x)
                       + sqr (ROBOT_CENTER.y - point.y))
                      <= sqr (constants::LinePoints::robot_radius);
      bool out_of_robot = ! in_robot;
      
      bool not_already_used = used_points.find (make_pair (point.x, point.y)) == used_points.end();
      
      if (out_of_robot && not_already_used) {
        points[j].push_back (point);
        used_points.insert (make_pair (point.x, point.y));
      }
    }
  }
}
