/**
 * @Filename FrameTransformations2D.hpp
 * @Description FrameTransformations2D class header
 * @Status Implementing
 * @Version $Id: FrameTransformations2D.hpp 5750 2012-07-05 01:50:42Z jreis $
 * @Maintainer Hugo Costelha
 */

#ifndef _FRAME_TRANSFORMATIONS_2D_HPP_
#define _FRAME_TRANSFORMATIONS_2D_HPP_

#include "Defines.hpp"



/**
 * @Class FrameTransformations2D FrameTransformations2D.hpp "FrameTransformations2D.hpp"
 * Class to transform local coordinates to world coordinates and vice-versa.
 * @Author Hugo Costelha
 */
class FrameTransformations2D
{
  public:
  
    /**
     * Constructor
     */
    FrameTransformations2D();
    
    /**
     * Destructor
     */
    ~FrameTransformations2D();
    
    /**
     * Transform polar coordinates to rectangular coordinates
     * @Argument distance - distance in polar coordinates
     * @Argument angle - angle in polar coordinates
     * @Argument x - value to be returned in rectangular coordinates
     * @Argument y - value to be returned in rectangular coordinates
     */
    static void polar2Rect (Distance distance, Angle angle, Distance* x, Distance* y);
    
    /**
     * Transform rectangular coordinates to polar coordinates
     * @Argument x - x value in rectangular coordinates
     * @Argument y - y value in rectangular coordinates
     * @Argument distance - distance value to be returned in polar coordinates
     * @Argument angle - angle value to be returned in polar coordinates
     */
    static void rect2Polar (Distance x, Distance y, Distance* distance, Angle* angle);
}; // class FrameTransformations2D
#endif
