/**
 * @Filename FrameTransformations3D.hpp
 * @Description FrameTransformations3D class header
 * @Status Implementing
 * @Version $Id: FrameTransformations3D.hpp 5750 2012-07-05 01:50:42Z jreis $
 * @Maintainer Hugo Costelha
 */

#ifndef _FRAME_TRANSFORMATIONS_3D_HPP_
#define _FRAME_TRANSFORMATIONS_3D_HPP_

#include "Defines.hpp"



/**
 * @Class FrameTransformations3D FrameTransformations3D.hpp "FrameTransformations3D.hpp"
 * Class to perform transformations of objects with x,y,z coordinates.
 * @Author Hugo Costelha
 */
class FrameTransformations3D
{
  public:
  
    /**
     * Constructor
     */
    FrameTransformations3D();
    
    /**
     * Destructor
     */
    ~FrameTransformations3D();
    
    /**
     * Transform spherical coordinates to cartesian coordinates
     * @Argument distance - distance in spherical coordinates
     * @Argument azimuth - angle from the x-axis in spherical coordinates
     * @Argument zenith - angle from the z-axis in spherical coordinates
     * @Argument x - value to be returned in cartesian coordinates
     * @Argument y - value to be returned in cartesian coordinates
     * @Argument z - value to be returned in cartesian coordinates
     */
    static void spherical2Cartesian (
      Distance distance, Angle azimuth, Angle zenith,
      Distance* x, Distance* y, Distance* z);
      
    /**
     * Transform cartesian coordinates to shperical coordinates
     * @Argument x - x value in cartesian coordinates
     * @Argument y - y value in cartesian coordinates
     * @Argument z - z value in cartesian coordinates
     * @Argument distance - distance value to be returned in shperical
     * coordinates
     * @Argument azimuth - angle from the x-axis in spherical coordinates
     * @Argument zenith - angle from the z-axis in spherical coordinates
     */
    static void cartesian2Spherical (
      Distance x, Distance y, Distance z,
      Distance* distance, Angle* azimuth, Angle* zenith);
}; // class FrameTransformations3D
#endif
