/**
 * @Filename MirrorImageFrameRobotFrameTransformations.cpp
 * @Description Transform image frame coordinates to robot frame coordinates
 * and vice-versa for the camera+mirror setup.
 * @Status Implementing
 * @Version $Id: MirrorImageFrameRobotFrameTransformations.cpp 5864 2012-12-08 14:40:54Z jreis $
 * @Maintainer Hugo Costelha
 */

#include "MirrorImageFrameRobotFrameTransformations.hpp"
#include <socrob/math.h>
using namespace socrob;



MirrorImageFrameRobotFrameTransformations::
MirrorImageFrameRobotFrameTransformations (float gain,
    int xRobotInImg,
    int yRobotInImg) :
  ImageFrameRobotFrameTransformations(),
  gain_ (gain)
{
  xRobotInImg_ = xRobotInImg;
  yRobotInImg_ = -yRobotInImg;
  
  // Assume for now that the camera is aligned with the robot
  yCamera_ = 0;
  xCamera_ = 0;
  float tmpXCamera;
  float tmpYCamera;
  // Get the robot position in field coordinates
  fromImageFrame2RobotFrame (xRobotInImg, yRobotInImg,
                             &tmpXCamera, &tmpYCamera);
  // The camera position is the opposite
  xCamera_ = -tmpXCamera;
  yCamera_ = -tmpYCamera;
  
  cameraDistance_ = sqrt (sqr (xCamera_) +
                          sqr (yCamera_));
}



MirrorImageFrameRobotFrameTransformations::~MirrorImageFrameRobotFrameTransformations()
{
}



void MirrorImageFrameRobotFrameTransformations::
fromImageFrame2RobotFrame (int imgX, int imgY,
                           float* robotX, float* robotY)
{
  *robotX = imgX * gain_ + xCamera_;
  *robotY = imgY * gain_ + yCamera_;
}



void MirrorImageFrameRobotFrameTransformations::
fromImageFrame2RobotFrameRaw (float imgDistance, float* robotDistance)
{
  *robotDistance = imgDistance * gain_;
}



void MirrorImageFrameRobotFrameTransformations::
fromRobotFrame2ImageFrame (float robotX, float robotY,
                           float* imgX, float* imgY)
{
  *imgX = (robotX - xCamera_) / gain_;
  *imgY = (robotY - yCamera_) / gain_;
}



// void MirrorImageFrameRobotFrameTransformations::
//         fromRobotFrame2ImageFrame(float robotDistance, float *imgDistance)
// {
//     *imgDistance = (robotDistance - cameraDistance_)/gain_;
// }



void MirrorImageFrameRobotFrameTransformations::
fromRobotFrame2ImageFrameRaw (float robotDistance, float* imgDistance)
{
  *imgDistance = robotDistance / gain_;
}
