#include <iostream>

#include <ros/ros.h>
#include <image_transport/image_transport.h>

#include <sensor_msgs/image_encodings.h>
#include <cv_bridge/cv_bridge.h>

#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>



using namespace std;
using namespace cv;



int
main (int argc,
      char** argv)
{
  ros::init (argc, argv, "image_pub");
  
  if (argc < 2) {
    cerr << "Please use " << argv[0] << " image_name i:=ros_topic" << endl;
    return -1;
  }
  
  ros::NodeHandle nh;
  image_transport::ImageTransport it (nh);
  
  ROS_INFO_STREAM ("Loading from " << argv[1]);
  
  sensor_msgs::Image::ConstPtr out_img;
  {
    Mat image;
    image = imread (argv[1], CV_LOAD_IMAGE_COLOR);
    if (! image.data) {
      cerr <<  "Could not open or find the image" << endl ;
      return -1;
    }
    
    cv_bridge::CvImage out_msg;
    out_msg.header.stamp = ros::Time::now();
    out_msg.encoding = sensor_msgs::image_encodings::BGR8;
    out_msg.image = image;
    out_img = out_msg.toImageMsg();
  }
  
  ROS_INFO_STREAM ("Publishing to " << ros::names::remap ("i"));
  
  image_transport::Publisher pub_ (it.advertise ("i", 1, false));
  
  ros::Rate rate (5);
  while (ros::ok()) {
    // Cannot latch the topic because compressed images do not get published
    // if there is no subscriber
    pub_.publish (out_img);
    rate.sleep();
  }
  
  return 0;
}
