/**
* @Filename Boolean.hpp
* Boolean Data class header.
* @Status Implementing
* @Version $Id: Boolean.hpp 5748 2012-07-04 17:44:30Z jreis $
* @Maintainer Joao Francisco (jrfe@mega.ist.utl.pt)
*/

#ifndef _Boolean_HPP_
#define _Boolean_HPP_
#include "Data.hpp"



/**
* @Class Boolean Boolean.hpp "Boolean.hpp"
* Class Boolean Data which represents all data that will be storable under the format Boolean.
* @Author Joao Francisco
*/
class Boolean : public Data
{
  public:
    Boolean (bool value = false) : Data(), value_ (value) {}
    
    Boolean (char const* dataString) {
      (*this) = Boolean (string (dataString));
    }
    
    Boolean (string const& dataString) :
      Data(),
      value_ (false) {
      if (dataString.find ("true") != string::npos) {
        value_ = true;
      }
    }
    
    bool boolValue() const {
      return value_;
    }
    void setValue (bool value) {
      value_ = value;
    }
    
    shared_ptr<Parameter> clone() const {
      return shared_ptr<Parameter> (new Boolean (value_));
    }
    string toString() const {
      return value_ ? "true" : "false";
    }
    
  private:
    bool value_;
};

#endif
