/**
* @Filename Data.cpp
* Data class implementation.
* @Status Implementing
* @Version $Id: Data.cpp 5661 2012-05-11 13:43:55Z jreis $
* @Maintainer Hugo Veiga (hasv@mega.ist.utl.pt)
*/

#include "Data.hpp"

#include "Parameter.hpp"

#include <Boolean.hpp>
#include <DataArray.hpp>
#include <DataMap.hpp>
#include <Float.hpp>
#include <FullPosture.hpp>
#include <Integer.hpp>
#include <Obstacle.hpp>
#include <Position2D.hpp>
#include <Position3D.hpp>
#include <Posture.hpp>
#include <String.hpp>
#include <TrackedPosition2D.hpp>
#include <TrackedPosition3D.hpp>
#include <TrackedPosture.hpp>
#include <Velocity2D.hpp>
#include <Velocity3D.hpp>
#include <VelocityPosture.hpp>



//#include <iostream>



Data::Data() : Parameter(),
  timeStamp(), isDifferentialDataType_ (false), name_ ("")
{
}



Data::Data (TimeStamp t) : Parameter(),
  timeStamp (t), isDifferentialDataType_ (false), name_ ("")
{
}



Data::~Data()
{
  //Nothing done yet
  //  std::cout << "Data::~Data()" << std::endl;
}



void Data::setDifferentialDataType (bool value)
{
  isDifferentialDataType_ = value;
}



bool Data::isDifferentialDataType()
{
  return isDifferentialDataType_;
}



//this method exists and it job is not done in a Data constructor because It is necessary to return an error when xml has errors
//! @TODO use exceptions and move this to a constructor
shared_ptr<Data> Data::buildData (SocRobXMLNode* currentNode, string& name)
{
  if (currentNode == 0) {
    return shared_ptr<Data>();
  }
  
  currentNode->getAtributeString ("Name", name);
  //         cout << "WorldInfo: \tData name:" << name << endl;
  
  string type;
  currentNode->getAtributeString ("Type", type);
  //         cout << "WorldInfo: \tData type:" << type << endl;
  
  string value;
  currentNode->getAtributeString ("Value", value);
  
  //         cout << "WorldInfo: \tValue:" << value << endl;
  
  // @TODO Data type creation should be done by a data factory using the XML node
  
  if (type == "Boolean") {
    shared_ptr<Boolean> b (new Boolean (value));
    //             cout << "WorldInfo: adding boolean:" << b->toString() << endl;
    return b;
  }
  else if (type == "DataMap") {
    shared_ptr<DataMap> d (new DataMap());
    //             cout << "WorldInfo: adding DataMap:" << d->toString();
    return d;
  }
  else if (type == "DataArray") {
    shared_ptr<DataArray> d (new DataArray (0));
    //             cout << "WorldInfo: adding DataMap:" << d->toString();
    return d;
  }
  else if (type == "Float") {
    shared_ptr<Float> f (new Float (value));
    //             cout << "WorldInfo: adding Float:" << f->toString() << endl;
    return f;
  }
  else if (type == "FullPosture") {
    shared_ptr<FullPosture> f (new FullPosture (value));
    //             cout << "WorldInfo: adding Float:" << f->toString() << endl;
    return f;
  }
  else if (type == "Integer") {
    shared_ptr<Integer> i (new Integer (value));
    //             cout << "WorldInfo: adding Integer:" << i->toString() << endl;
    return i;
  }
  else if (type == "Obstacles") {
    shared_ptr<Obstacles> o (new Obstacles());
    //             cout << "WorldInfo: adding Obstacles" << endl;
    return o;
  }
  else if (type == "Posture") {
    shared_ptr<Posture> p (new Posture (value));
    return p;
  }
  else if (type == "Position2D") {
    shared_ptr<Position2D> p (new Position2D (value));
    return p;
  }
  else if (type == "Position3D") {
    shared_ptr<Position3D> p (new Position3D (value));
    return p;
  }
  else if (type == "String") {
    shared_ptr<String> s (new String (value));
    //             cout << "WorldInfo: adding String:" << s->toString() << endl;
    return s;
  }
  else if (type == "Velocity2D") {
    shared_ptr<Velocity2D> v (new Velocity2D (value));
    //             cout << "WorldInfo: adding Velocity:" << v->toString() << endl;
    return v;
  }
  else if (type == "Velocity3D") {
    shared_ptr<Velocity3D> v (new Velocity3D (value));
    //             cout << "WorldInfo: adding Velocity:" << v->toString() << endl;
    return v;
  }
  else if (type == "VelocityPosture") {
    shared_ptr<VelocityPosture> v (new VelocityPosture (value));
    //             cout << "WorldInfo: adding Velocity:" << v->toString() << endl;
    return v;
  }
  /*else if(type == "TimedData")
  {
      SocRobXMLNode valueNode;
      currentNode->getNextSubNode("Value", &valueNode);
      SocRobXMLNode dataNode;
      valueNode.getNextSubNode("Data", &dataNode);
      TimedData * t = new TimedData(&dataNode);
      return t;
  }*/
  else if (type == "TrackedPosition2D") {
    /// @TODO: Allow non-zero initialization of 2D tracked positions
    shared_ptr<TrackedPosition2D> t (new TrackedPosition2D());
    //             cout << "WorldInfo: adding Float:" << f->toString() << endl;
    return t;
  }
  else if (type == "TrackedPosition3D") {
    /// @TODO: Allow non-zero initialization of 3D tracked positions
    shared_ptr<TrackedPosition3D> t (new TrackedPosition3D());
    //             cout << "WorldInfo: adding Float:" << f->toString() << endl;
    return t;
  }
  else if (type == "TrackedPosture") {
    /// @TODO: Allow non-zero initialization of tracked postures
    shared_ptr<TrackedPosture> t (new TrackedPosture());
    //             cout << "WorldInfo: adding Float:" << f->toString() << endl;
    return t;
  }
  else {
    cerr << "Data::buildData: unrecognized data type: " << type << endl;
  }
  
  return shared_ptr<Data>();
}



Data::Data (const Data& oldData) : Parameter (oldData),
  timeStamp (oldData.timeStamp),
  isDifferentialDataType_ (oldData.isDifferentialDataType_),
  name_ (oldData.name_)
{
}



Data& Data::operator= (const Data& oldData)
{
  // Copy the parameter part
  Parameter::operator= (oldData);
  
  // Fill the clone values with copies of our values.
  isDifferentialDataType_ = oldData.isDifferentialDataType_;
  //     name_ = name_;
  timeStamp = oldData.timeStamp;
  return *this;
}
