/**
* @Filename DataArray.hpp
* DataArray class header.
* @Status Implementing
* @Version $Id: DataArray.hpp 5750 2012-07-05 01:50:42Z jreis $
* @Maintainer Hugo Veiga (hasv@mega.ist.utl.pt)
*/

#ifndef _DATAARRAY_HPP_
#define _DATAARRAY_HPP_

#include <vector>
#include <set>
#include <stdexcept>
#include "Data.hpp"



/**
* @Class DataArray DataArray.hpp "DataArray.hpp"
* Class DataArray which stores TimeData for a slot
* @Author Hugo Veiga
*/
class DataArray : public Data
{
  public:
    /**
     * Constructor.
     * This method creates a new DataArray (no need to declare max number of elements).
     * @Argument numMaxElements - ?
     */
    DataArray();
    /**
     * Constructor.
     * This method creates a new DataArray.
     * @Argument numMaxElements - ?
     */
    DataArray (int numMaxElements);
    
    /**
     * Destructor.
     * This method makes deinitializations for objects of this class.
     */
    virtual ~DataArray();
    
    /**
     * This method adds a new TimedData to the current DataArray.
     * @Argument data - with time stamp.
     * @Returns void.
     */
    void addElement (shared_ptr<Data> data);
    
    /**
     * This method retrieves a TimedData from the current DataArray
     * @Argument index - index (by cronologic order) of the element to retrieve.
     * @Returns Reference for a TimedData, or 0 otherwise.
     */
    shared_ptr<Data> getElement (int index);
    shared_ptr<Data const> getElement (int index) const;
    
    /**
     * This method retrieves the most recent TimedData added to the current DataArray.
     * @Returns Reference for a TimedData, or 0 otherwise.
     */
    shared_ptr<Data> getMostRecentElement();
    
    /**
     * This method return the number of Data items in vector
     * @Returns Size of the current DataArray
     */
    int getSize() const;
    
    /**
     * ?
     */
    int getNumMaxElements();
    
    /**
     * allocate more space for elements..
     */
    void incNumMaxElements (int numOfElements2Add);
    
    static
    shared_ptr<DataArray>
    from (const set<string>& s);
    
    
    /**
     * ?
     */
    virtual shared_ptr<Parameter> clone() const;
    
    /**
     * ?
     */
    string toString() const;
    
    
  protected:
  private:
    std::vector<shared_ptr<Data> > dataArray_; ///< Stores the TimedData's.
    int actualPosition_;
    int numMaxElements_;
    int size_;
    bool dynamicArray_;
};

#endif
