/**
* @Filename DataMap.cpp
* DataMap class implementation.
* @Status Implementing
* @Version $Id: DataMap.cpp
* @Maintainer Nelson Ramos nmsra@mega.ist.utl.pt
*/

#include "DataMap.hpp"

#include <sstream>



DataMap::DataMap() : Data(),
  dataMap_(), size_ (0)
{
}



DataMap::~DataMap()
{
  //  std::cout << "DataMap::~DataMap()" << std::endl;
  dataMap_.clear();
}



void DataMap::addElement (string dataName, shared_ptr<Data> data)
{
  //  std::cout << "DataMap::addElement : dataName=" << dataName << std::endl;
  if (hasElement (dataName)) {
    //cout << "WARNING: DataMap::addElement : You will overwrite dataName = " << dataName << endl;
    // I wont miss this:
    // delete dataMap_[dataName];
  }
  else {
    size_++;
  }
  
  dataMap_[dataName] = data;
}



bool DataMap::removeElement (string dataName)
{
  //  std::cout << "DataMap::removeElement : dataName=" << dataName << std::endl;
  dataMap_t::iterator it = dataMap_.find (dataName);
  
  if (it != dataMap_.end()) {
    dataMap_.erase (it);
    return true;
  }
  else {
    return false;
  }
}



bool DataMap::removeAllElements()
{
  //  std::cout << "DataMap::removeAllElements" << std::endl;
  dataMap_.clear();
  return true;
}



shared_ptr<Data> DataMap::getElement (string dataName)
{
  return dataMap_[dataName];
}



shared_ptr<Data const> DataMap::getElement (string dataName) const
{
  dataMap_t::const_iterator it = dataMap_.find (dataName);
  if (it != dataMap_.end()) {
    return it->second;
  }
  
  return shared_ptr<Data const>();
}



bool DataMap::hasElement (string dataName) const
{
  return (dataMap_.find (dataName) != dataMap_.end());
}



shared_ptr<DataArray> DataMap::getArray()
{
  int size = getSize();
  shared_ptr<DataArray> ret (new DataArray (size));
  
  dataMap_t::iterator it = dataMap_.begin();
  dataMap_t::iterator end = dataMap_.end();
  for (; it != end; it++) {
    ret->addElement (it->second);
  }
  return ret;
}



//HACK: to resolve after Paderne
//very Carefull with this function..does not delete its objects
DataMap::dataMap_t DataMap::mapValue()
{
  return dataMap_;
}



shared_ptr<Parameter> DataMap::clone() const
{
  shared_ptr<DataMap> ret (new DataMap());
  
  dataMap_t::const_iterator it;
  for (it = dataMap_.begin(); it != dataMap_.end(); it++) {
    ret->addElement (it->first, dynamic_pointer_cast<Data> (it->second->clone()));
  }
  
  return ret;
}



string DataMap::toString() const
{
  ostringstream s;
  s << "[";
  bool isFirst = true;
  typedef std::pair<string, shared_ptr<Data> > pabtype;
  foreach (pabtype pab, dataMap_) {
    if (!isFirst) {
      s << ",";
    }
    s << " (" << pab.first << ":" << pab.second->toString() << ")";
    isFirst = false;
  }
  s << " ]";
  return s.str();
  
#if 0
  stringstream ret;
  
  //     int size = getSize();
  int count = 1;
  
  dataMap_t::const_iterator it;
  for (it = dataMap_.begin(); it != dataMap_.end(); it++) {
    if (! (count == 1)) {
      ret << ":";
    }
    
    shared_ptr<Data> d = it->second;
    ret << d->toString();
    count++;
  }
  
  return ret.str();
#endif
}



int DataMap::getSize()
{
  return size_;
}
