/**
* @Filename DataMap.hpp
* DataArray class header.
* @Status Implementing
* @Version $Id: DataMap.hpp
* @Maintainer Nelson Ramos (nmsra@mega.ist.utl.pt)
*/

#ifndef _DATAMAP_HPP_
#define _DATAMAP_HPP_

#include <vector>
#include <stdexcept>
#include "Data.hpp"
#include "DataArray.hpp"

#include <map>



using namespace std;



/**
* @Class DataMap DataMap.hpp "DataMap.hpp"
* Class DataMap which stores Data
* @Author Nelson Ramos
*/
class DataMap : public Data
{
  public:
  
  
    typedef map<string, shared_ptr<Data> > dataMap_t;
    
    
    
    /**
     * Constructor.
     * This method creates a new DataMap.
     */
    DataMap();
    
    /**
     * Destructor.
     * This method makes deinitializations for objects of this class.
     */
    virtual ~DataMap();
    
    /**
     * This method adds a new Data to the current DataMap.
     * @Argument dataName - ?
     * @Argument data - ?
     * @Returns void.
     */
    void addElement (string dataName, shared_ptr<Data> data);
    
    /**
     *  This method retTimedrieves a TimedData from the current DataArray
     * @Argument dataName - index (by cronologic order) of the element to retrieve.
     * @Returns Reference for a TimedData, or 0 otherwise.
     */
    shared_ptr<Data> getElement (string dataName);
    shared_ptr<Data const> getElement (string dataName) const;
    
    
    /**
     * This methods tells if the map already has an element with the specified key
     * @Argument dataName - ?
     */
    bool hasElement (string dataName) const;
    
    
    bool removeElement (string dataName);
    
    bool removeAllElements();
    
    
    /**
     * This method return the number of Data items in vector
     * @Returns Size of the current DataArray
     */
    int getSize();
    
    /**
     * This method builds an array with all the elements that are in the DataMap
     */
    shared_ptr<DataArray> getArray();
    
    /**
     * Returns map value. This is a major hack!
     * @TODO remove this function.
     */
    dataMap_t mapValue();
    
    /**
     * ?
     */
    shared_ptr<Parameter> clone() const;
    
    /**
     * ?
     */
    string toString() const;
    
    
    typedef dataMap_t::iterator dataMapIterator_t;
    
  protected:
  private:
    dataMap_t dataMap_;
    int size_;
};

#endif
