/**
* @Filename Float.hpp
* Float Data class header.
* @Status Implementing
* @Version $Id: Float.hpp 5748 2012-07-04 17:44:30Z jreis $
* @Maintainer Joao Francisco (jrfe@mega.ist.utl.pt)
*/

#ifndef _FLOAT_HPP_
#define _FLOAT_HPP_
#include "Data.hpp"



/**
* @Class Float Float.hpp "Float.hpp"
* Class Float Data which represents all data that will be storable under the format Float.
* @Author Joao Francisco
*/
class Float : public Data
{
  public:
    /**
     * Constructor.
     * This method creates a new Float.
     */
    Float (void);
    
    /**
     * Constructor.
     * This method creates a new DataArray.
     * @Argument stringData - ?
     */
    Float (string stringData);
    Float (char const* dataString) {
      (*this) = Float (string (dataString));
    }
    
    /**
     * Constructor.
     * This method creates a new Float.
     * @Argument value - to be store by this Float.
     */
    Float (float value);
    
    /**
     * This method retrieves the value held by this Float.
     * @Returns value held by this Data, 0.0 if uninitialized.
     */
    float floatValue() const;
    
    /**
     * ?
     * @Argument value - ?
     */
    void setValue (float value);
    
    /**
     * Destructor.
     * This method makes deinitializations for objects of this class.
     */
    ~Float (void);
    
    /**
     * This method create a new object that is a clone of this
     * @Returns clone Parameter
     */
    shared_ptr<Parameter> clone() const;
    
    /**
     * cast operations
     * @return Float::value_
     * @Author Nelson Ramos
     */
    string toString() const;
    
    
  private:
    float value_;
};
#endif
