/**
 * @Filename FullPosture.cpp
 * @Description Vector posture class implementation
 * @Status Finished
 * @Version $Id: FullPosture.cpp 5661 2012-05-11 13:43:55Z jreis $
 * @Maintainer Hugo Costelha
 */

#include <FullPosture.hpp>

#include <LocalFrameWorldFrameTransformations.hpp>
using namespace LocalFrameWorldFrameTransformations;



FullPosture::FullPosture() : Data(),
  worldPosture(), localVelocity(), worldVelocity(), isLocalized (false)
{
}

#include <cstring>
#include <cstdlib>
#include <stack>



FullPosture::FullPosture (string dataString) :
  worldPosture(),
  localVelocity(),
  worldVelocity(),
  isLocalized()
{
  char* data, * token;
  data = strdup (dataString.c_str());
  float inX = 0.0, inY = 0.0, inTheta = 0.0;
  float inVX = 0.0, inVY = 0.0, inW = 0.0;
  bool newVelocityInLocalCoordinates = true;
  bool isLocalized = false;
  
  stack<char*> tokens;
  
  token = strtok (data, "(), ");
  while (token) {
    tokens.push (token);
    token = strtok (NULL, "(), ");
  }
  
  switch (tokens.size()) {
    case 8:
      isLocalized = (0 == strcmp (tokens.top(), "true"));
      tokens.pop();
    case 7:
      newVelocityInLocalCoordinates = (0 == strcmp (tokens.top(), "true"));
      tokens.pop();
    case 6:
      sscanf (tokens.top(), "%f", &inW);
      tokens.pop();
    case 5:
      sscanf (tokens.top(), "%f", &inVY);
      tokens.pop();
    case 4:
      sscanf (tokens.top(), "%f", &inVX);
      tokens.pop();
    case 3:
      sscanf (tokens.top(), "%f", &inTheta);
      tokens.pop();
    case 2:
      sscanf (tokens.top(), "%f", &inY);
      tokens.pop();
    case 1:
      sscanf (tokens.top(), "%f", &inX);
      tokens.pop();
    case 0:
      break;
    default:
      //KAPOOT
      break;
  }
  
  worldPosture = Posture (inX, inY, inTheta, true);
  VelocityPosture newVelocity (inVX, inVY, inW, true);
  if (newVelocityInLocalCoordinates) {
    localVelocity =  newVelocity;
    // Not get the world velocity from the local velocity
    local2World (worldPosture, localVelocity, worldVelocity);
    worldVelocity.timeStamp.setTimeStamp (localVelocity.timeStamp);
  }
  else {
    worldVelocity = newVelocity;
    // Now get the local velocity from the world velocity
    world2Local (worldPosture, worldVelocity, localVelocity);
    localVelocity.timeStamp.setTimeStamp (worldVelocity.timeStamp);
  }
  
  this->isLocalized = isLocalized;
  free (data);
}



FullPosture::FullPosture (Posture newWorldPosture,
                          VelocityPosture newVelocity,
                          bool newVelocityInLocalCoordinates) : Data(),
  worldPosture (newWorldPosture), localVelocity(), worldVelocity(),
  isLocalized (false)
{
  if (newVelocityInLocalCoordinates) {
    localVelocity =  newVelocity;
    // Not get the world velocity from the local velocity
    local2World (worldPosture, localVelocity, worldVelocity);
    worldVelocity.timeStamp.setTimeStamp (localVelocity.timeStamp);
  }
  else {
    worldVelocity = newVelocity;
    // Now get the local velocity from the world velocity
    world2Local (worldPosture, worldVelocity, localVelocity);
    localVelocity.timeStamp.setTimeStamp (worldVelocity.timeStamp);
  }
}



shared_ptr<Parameter> FullPosture::clone() const
{
  shared_ptr<FullPosture> newRobot (new FullPosture (*this));
  return newRobot;
}



string FullPosture::toString() const
{
  if (isLocalized) {
    return worldPosture.toString() + localVelocity.toString() + "(true)";
  }
  else {
    return worldPosture.toString() + localVelocity.toString() + "(false)";
  }
}
