/**
 * @Filename Obstacle.cpp
 * @Description ...to be described...
 * @Status unknown
 * @Version $Id: Obstacle.cpp 5864 2012-12-08 14:40:54Z jreis $
 * @Maintainer Nelson Ramos (nmsra@mega.ist.utl.pt)
 */

#include "Obstacle.hpp"

#include <socrob/math.h>
using namespace socrob;



Obstacle::
Obstacle() :
  Data(),
  distance_ (INFTY),
  angle_ (0),
  velocity_()
{
}



Obstacle::
Obstacle (Distance distance,
          Angle angle,
          Velocity2D velocity) :
  Data(),
  distance_ (distance),
  angle_ (angle),
  velocity_ (velocity)
{
}



Distance Obstacle::d() const
{
  return getDistance();
}



Angle Obstacle::th() const
{
  return getAngle();
}



Distance  Obstacle::getDistance() const
{
  return distance_;
}



Angle Obstacle::getAngle() const
{
  return angle_;
}



void Obstacle::setDistance (Distance d)
{
  distance_ = d;
}



void Obstacle::setAngle (Angle th)
{
  angle_ = th;
}



void Obstacle::setVelocity (Velocity2D v)
{
  velocity_ = v;
}



Velocity2D Obstacle::getVelocity()
{
  return velocity_;
}



shared_ptr<Parameter> Obstacle::clone() const
{
  shared_ptr<Obstacle> newObstacle (new Obstacle (*this));
  return newObstacle;
}



string Obstacle::toString() const
{
  //cout << "[d,theta(deg)] = [" << distance_ << "," << angle_ << "]";
  
  char buf[15];
  string ret = "";
  ret += "[d,theta(deg)] = [";
  sprintf (buf, "%.8f", (float) distance_);
  ret += buf;
  ret += ", ";
  sprintf (buf, "%.8f", (float) angle_);
  ret += buf;
  ret += "]";
  
  return  ret;
}



Obstacles::
Obstacles (int n) :
  DataArray (n)
{
}



Obstacle const& Obstacles::getObstacle (int index) const
{
  return * (dynamic_pointer_cast<Obstacle const> (getElement (index)));
}



int Obstacles::getSize() const
{
  return DataArray::getSize();
}



shared_ptr<Parameter> Obstacles::clone()
{
  int size = getSize();
  shared_ptr<Obstacles> ret (new Obstacles (size));
  for (int i = 0; i < size; i++) {
    shared_ptr<Data> o = getElement (i);
    ret->addElement (dynamic_pointer_cast<Data> (o->clone()));
  }
  return ret;
}
