/**
 * @Filename Obstacle.hpp
 * @Description ...to be described...
 * @Status unknown
 * @Version $Id: Obstacle.hpp 5671 2012-05-16 13:26:42Z jreis $
 * @Maintainer Nelson Ramos (nmsra@mega.ist.utl.pt)
 */

#ifndef _OBSTACLE_HPP_
#define _OBSTACLE_HPP_

#include <iostream>
#include <string>
#include "Defines.hpp"
#include "DataArray.hpp"
#include "Velocity2D.hpp"

#include <socrob_ros/Obstacle.h>
#include <socrob_ros/Obstacles.h>



using namespace std;



/**
 * @Class Obstacle Obstacle.hpp "Obstacle.hpp"
 * ?
 * @Author Nelson Ramos
 */
class Obstacle : public Data
{
  public:
    Obstacle();
    Obstacle (Distance distance, Angle angle, Velocity2D velocity = Velocity2D (0, 0));
    Obstacle (Obstacle* obstacle);
    ~Obstacle() {}
    
    Distance d() const;
    Angle th() const;
    Distance  getDistance() const;
    Angle getAngle() const;
    Velocity2D getVelocity();
    
    void setDistance (Distance d);
    void setAngle (Angle th);
    void setVelocity (Velocity2D v);
    
    shared_ptr<Parameter> clone() const;
    string toString() const;
    
    void rosMsg (socrob_ros::Obstacle& msg) const {
      msg.angle = angle_;
      msg.distance = distance_;
      velocity_.rosMsg (msg.velocity);
    }
    
    void fromRosMsg (const socrob_ros::Obstacle& msg) {
      angle_ = msg.angle;
      distance_ = msg.distance;
      velocity_.fromRosMsg (msg.velocity);
    }
    
    static shared_ptr<Obstacle> newFromRosMsg (const socrob_ros::Obstacle& msg) {
      shared_ptr<Obstacle> tmp = shared_ptr<Obstacle> (new Obstacle ());
      tmp->fromRosMsg (msg);
      return tmp;
    }
    
  private:
    Distance distance_;
    Angle angle_;
    Velocity2D velocity_; ///< linear velocity of the obstacle.
};



/**
 * @Class Obstacles Obstacle.hpp "Obstacle.hpp"
 * ?
 * @Author Nelson Ramos
 */
class Obstacles : public DataArray
{
  public:
    Obstacles (int n = 0);
    
    Obstacle const& getObstacle (int n) const;
    int getSize() const;
    
    virtual shared_ptr<Parameter> clone();
    
    void rosMsg (socrob_ros::Obstacles& msg) const {
      for (int i = 0; i < getSize(); i++) {
        socrob_ros::Obstacle o;
        getObstacle (i).rosMsg (o);
        msg.obstacles.push_back (o);
      }
    }
    
    static shared_ptr<Obstacles> newFromRosMsg (const socrob_ros::Obstacles& msg) {
      shared_ptr<Obstacles> tmp = shared_ptr<Obstacles> (new Obstacles (msg.obstacles.size()));
      foreach (socrob_ros::Obstacle const & o, msg.obstacles) {
        tmp->addElement (Obstacle::newFromRosMsg (o));
      }
      return tmp;
    }
};

#endif
