/**
 * @Filename Posture.cpp
 * @Description  Posture class implementation
 * @Status Finished
 * @Version $Id: Posture.cpp 5864 2012-12-08 14:40:54Z jreis $
 * @Maintainer Hugo Costelha
 */

#include <Posture.hpp>
#include <math.h>

#include <socrob/math.h>
using namespace socrob;

#include <sstream>



Posture::Posture() : Data(),
  pos2D_(), theta_ (0.0), uncertainty_ (0.0), trustTheta_ (1.0)
{
}



Posture::Posture (string dataString) : Data(),
  pos2D_(), theta_(), uncertainty_(), trustTheta_()
{
  const char* data;
  data = dataString.c_str();
  float inX = 0.0, inY = 0.0, inTheta = 0.0;
  
  sscanf (data, "(%f,%f,%f)", &inX, &inY, &inTheta);
  
  setXYTheta (inX, inY, inTheta);
}



Posture::Posture (float x, float y, float theta, bool isRect) : Data(),
  pos2D_ (x, y, isRect), theta_ (theta), uncertainty_ (0.0), trustTheta_ (1.0)
{
}



Posture::~Posture()
{
}



Distance Posture::getX() const
{
  return pos2D_.getX();
}



Distance Posture::getY() const
{
  return pos2D_.getY();
}



Distance Posture::getDistance() const
{
  return pos2D_.getDistance();
}



Angle Posture::getAzimuth() const
{
  return pos2D_.getAzimuth();
}



Angle Posture::getTheta() const
{
  return theta_;
}



void Posture::setPolar (Distance new_rho, Angle new_alpha, Angle new_theta)
{
  setPolar (new_rho, new_alpha);
  setTheta (new_theta);
}



void Posture::setPolar (Distance new_rho, Angle new_alpha)
{
  pos2D_.setPolar (new_rho, new_alpha);
}



void Posture::setDistance (Distance new_rho)
{
  pos2D_.setDistance (new_rho);
}



void Posture::setAzimuth (Angle new_alpha)
{
  pos2D_.setAzimuth (new_alpha);
}



void Posture::setXYTheta (Distance new_x, Distance new_y, Distance new_theta)
{
  setXY (new_x, new_y);
  setTheta (new_theta);
}



void Posture::setXY (Distance new_x, Distance new_y)
{
  pos2D_.setXY (new_x, new_y);
}



void Posture::setX (Distance new_x)
{
  pos2D_.setX (new_x);
}



void Posture::setY (Distance new_y)
{
  pos2D_.setY (new_y);
}



void Posture::setTheta (Angle new_theta)
{
  theta_ = clipAngle (new_theta);
}



void Posture::setUncertainty (float value)
{
  uncertainty_ = value;
}



void Posture::setTrust (float trustX, float trustY, float trustTheta)
{
  pos2D_.setTrust (trustX, trustY);
  trustTheta_ = trustTheta;
}



float Posture::getTrustX() const
{
  return pos2D_.getTrustX();
}



float Posture::getTrustY() const
{
  return pos2D_.getTrustY();
}



float Posture::getTrustTheta() const
{
  return trustTheta_;
}



float Posture::getUncertainty() const
{
  return uncertainty_;
}



shared_ptr<Parameter> Posture::clone() const
{
  shared_ptr<Posture> newPosture (new Posture (*this));
  return newPosture;
}



string Posture::toString() const
{
  stringstream ret;
  ret << "(" << getX() << "," << getY() << "," << getTheta() << ")";
  
  return ret.str();
}



//eof
