/**
* @Filename String.hpp
* String Data class header.
* @Status Implementing
* @Version $Id: String.hpp 5721 2012-06-15 17:09:39Z jreis $
* @Maintainer Joao Francisco (jrfe@mega.ist.utl.pt)
*/

#ifndef _STRING_HPP_
#define _STRING_HPP_
#include "Data.hpp"
#include <string>

#include "Parameter.hpp"



/**
* @Class String String.hpp "String.hpp"
* Class String Data which represents all data that will be storable under the format String.
* @Author Joao Francisco
*/
class String : public Data
{
  public:
    /**
     * Constructor.
     * This method creates a new String.
     */
    String();
    
    /**
     * Constructor.
     * This method creates a new String.
     * @Argument value - to be store by this String.
     */
    String (std::string value);
    
    /**
     * This method retrieves the value held by this String.
     * @Returns value held by this Data, empty string if uninitialized.
     */
    std::string stringValue() const;
    
    /**
     * ?
     * @Argument value - ?
     */
    void setValue (std::string value);
    
    /**
     * Destructor.
     * This method makes deinitializations for objects of this class.
     */
    ~String();
    
    /**
    * This method create a new object that is a clone of this
    * @Returns clone Parameter
    */
    
    shared_ptr<Parameter> clone() const;
    
    /**
     * cast operations
     * @return String::value
     * @Author Nelson Ramos
     */
    string toString() const;
    
  private:
    std::string value_;
};

#endif
