/**
 * @Filename TrackedPosition2D.cpp
 * TrackedPosition2D Data class implementation.
 * @Status Implementing
 * @Version $Id: TrackedPosition2D.cpp 5661 2012-05-11 13:43:55Z jreis $
 * @Maintainer Hugo Costelha
 */

#include <TrackedPosition2D.hpp>

#include <LocalFrameWorldFrameTransformations.hpp>
using namespace LocalFrameWorldFrameTransformations;



TrackedPosition2D::TrackedPosition2D() : Data(),
  localPosition(), localVelocity(), baseFrame(), worldPosition(),
  worldVelocity(), seeTrackedObject (false)
{
}



TrackedPosition2D::TrackedPosition2D (FullPosture newBaseFrame,
                                      Position2D newPosition,
                                      Velocity2D newVelocity,
                                      bool newSeeTrackedObject,
                                      bool inLocalCoordinates) : Data(),
  localPosition(), localVelocity(), baseFrame (newBaseFrame), worldPosition(),
  worldVelocity(), seeTrackedObject (newSeeTrackedObject)
{
  if (inLocalCoordinates) {
    localPosition = newPosition;
    localVelocity = newVelocity;
    // Get world coordinates from the local ones
    // Position
    local2World (baseFrame.worldPosture, localPosition, worldPosition);
    worldPosition.timeStamp.setTimeStamp (localPosition.timeStamp);
    // Velocity
    local2World (baseFrame.worldPosture, localPosition, baseFrame.localVelocity,
                 localVelocity, worldVelocity);
    worldVelocity.timeStamp.setTimeStamp (localVelocity.timeStamp);
  }
  else {
    worldPosition = newPosition;
    worldVelocity = newVelocity;
    // Get local coordinates from the world ones
    // Position
    world2Local (baseFrame.worldPosture, worldPosition, localPosition);
    localPosition.timeStamp.setTimeStamp (worldPosition.timeStamp);
    // Velocity
    world2Local (baseFrame.worldPosture, localPosition, baseFrame.localVelocity,
                 worldVelocity, localVelocity);
    localVelocity.timeStamp.setTimeStamp (worldVelocity.timeStamp);
  }
}



shared_ptr<Parameter> TrackedPosition2D::clone() const
{
  shared_ptr<TrackedPosition2D> newBall (new TrackedPosition2D (*this));
  return newBall;
}



string TrackedPosition2D::toString() const
{
  return localPosition.toString() + localVelocity.toString() +
         baseFrame.toString();
}
