/**
 * @Filename TrackedPosition2D.hpp
 * TrackedPosition2D Data class header.
 * @Status Implementing
 * @Version $Id: TrackedPosition2D.hpp 5750 2012-07-05 01:50:42Z jreis $
 * @Maintainer Hugo Costelha
 */

#ifndef _TRACKED_POSITION_2D_HPP_
#define _TRACKED_POSITION_2D_HPP_

#include <Data.hpp>
#include <Position2D.hpp>
#include <Velocity2D.hpp>
#include <FullPosture.hpp>



/**
 * @Class TrackedPosition2D TrackedPosition2D.hpp "TrackedPosition2D.hpp"
 * This class is used to aggregate all the information regarding a tracked
 * object in a 2D space. The observer is typically a robot, having a full
 * posture reference frame.
 * @Author Hugo Costelha
 */
class TrackedPosition2D : public Data
{
  public:
    /**
     * Constructor.
     * Everything set to 0/false.
     */
    TrackedPosition2D();
    
    /**
     * Constructor.
     * @Argument newBaseFrame - base frame info;
     * @Argument newPosition - ball position in local or world coordinates;
     * @Argument newVelocity - ball velocity in local or world coordinates;
     * @Argument newSeeTrackedObject - if true, then the tracked object was
     * observed, otherwise it was etimated indirectly from other sources.
     * @Argument inLocalCoordinates - (optional, true by default) if true, then
     * the given position and velocity are in local coordinates, otherwise they
     * are in world coordinates.
     */
    TrackedPosition2D (FullPosture newBaseFrame, Position2D newPosition,
                       Velocity2D newVelocity, bool newSeeTrackedObject,
                       bool inLocalCoordinates = true);
                       
    shared_ptr<Parameter> clone() const;
    string toString() const;
    
    Position2D localPosition; ///< Position of the ball in local coordinates.
    Velocity2D localVelocity; ///< Velocity of the ball in local coordinates
    FullPosture baseFrame; ///< Base frame info
    Position2D worldPosition; ///< Position of the ball in world coordinates.
    Velocity2D worldVelocity; ///< Velocity of the ball in world coordinates
    bool seeTrackedObject; ///< If true, the tracked objects was observed,
    ///< otherwise it was estimated from other sources.
};

#endif
