/**
 * @Filename TrackedPosition3D.cpp
 * TrackedPosition3D Data class implementation.
 * @Status Implementing
 * @Version $Id: TrackedPosition3D.cpp 5353 2011-11-11 05:52:42Z jreis $
 * @Maintainer Hugo Costelha
 */

#include <TrackedPosition3D.hpp>

#include <LocalFrameWorldFrameTransformations.hpp>
using namespace LocalFrameWorldFrameTransformations;



TrackedPosition3D::
TrackedPosition3D() :
  Data(),
  localPosition(),
  localVelocity(),
  baseFrame(),
  worldPosition(),
  worldVelocity(),
  seeTrackedObject (false),
  seeTrackedObjectLocally(),
  seeTrackedObjectBySharing()
{
}



TrackedPosition3D::
TrackedPosition3D (FullPosture newBaseFrame,
                   Position3D newPosition,
                   Velocity3D newVelocity,
                   bool newSeeTrackedObject,
                   bool inLocalCoordinates) :
  Data(),
  localPosition(),
  localVelocity(),
  baseFrame (newBaseFrame),
  worldPosition(),
  worldVelocity(),
  seeTrackedObject (newSeeTrackedObject),
  seeTrackedObjectLocally(),
  seeTrackedObjectBySharing()
{
  if (inLocalCoordinates) {
    localPosition = newPosition;
    localVelocity = newVelocity;
    // Get world coordinates from the local ones
    // Position
    local2World (baseFrame.worldPosture, localPosition, worldPosition);
    worldPosition.timeStamp.setTimeStamp (localPosition.timeStamp);
    // Velocity
    //         Velocity3D tempVelocity = localVelocity;
    //         tempVelocity.setVxVyVz(tempVelocity.getVx-baseFrame.localVelocity.getAngularSpeed()*localPosition.getY(), tempVelocity.getVy+baseFrame.localVelocity.getAngularSpeed()*localPosition.getX(), tempVelocity.getVz());
    //         local2World( baseFrame.worldPosture, baseFrame.localVelocity,
    //                     localVelocity, &worldVelocity );
    local2World (baseFrame.worldPosture, localPosition, baseFrame.localVelocity,
                 localVelocity, worldVelocity);
    worldVelocity.timeStamp.setTimeStamp (localVelocity.timeStamp);
  }
  else {
    worldPosition = newPosition;
    worldVelocity = newVelocity;
    // Get local coordinates from the world ones
    // Position
    world2Local (baseFrame.worldPosture, worldPosition, localPosition);
    localPosition.timeStamp.setTimeStamp (worldPosition.timeStamp);
    // Velocity
    world2Local (baseFrame.worldPosture, localPosition, baseFrame.localVelocity,
                 worldVelocity, localVelocity);
    localVelocity.timeStamp.setTimeStamp (worldVelocity.timeStamp);
  }
}



shared_ptr<Parameter>
TrackedPosition3D::
clone() const
{
  shared_ptr<TrackedPosition3D> newBall (new TrackedPosition3D (*this));
  return newBall;
}



string
TrackedPosition3D::
toString() const
{
  return localPosition.toString() + localVelocity.toString() +
         baseFrame.toString() + worldPosition.toString();
}
