/**
 * @Filename TrackedPosture.cpp
 * @Description Vector posture class implementation
 * @Status Finished
 * @Version $Id: TrackedPosture.cpp 5661 2012-05-11 13:43:55Z jreis $
 * @Maintainer Hugo Costelha
 */

#include <TrackedPosture.hpp>

#include <LocalFrameWorldFrameTransformations.hpp>
using namespace LocalFrameWorldFrameTransformations;



TrackedPosture::TrackedPosture() : Data(),
  baseFrame(), localPosture(), worldPosture(), localVelocity(),
  worldVelocity(), seeTrackedObject (false)
{
}



TrackedPosture::TrackedPosture (FullPosture newBaseFrame, Posture newPosture,
                                VelocityPosture newVelocity,
                                bool newSeeTrackedObject,
                                bool inLocalCoordinates) : Data(),
  baseFrame (newBaseFrame), localPosture(), worldPosture(), localVelocity(),
  worldVelocity(), seeTrackedObject (newSeeTrackedObject)
{
  if (inLocalCoordinates) {
    localPosture = newPosture;
    localVelocity =  newVelocity;
    // Not get the world coordinares from the local ones
    // Posture
    local2World (baseFrame.worldPosture, localPosture, worldPosture);
    worldPosture.timeStamp.setTimeStamp (localPosture.timeStamp);
    // Velocity
    local2World (baseFrame.worldPosture, localPosture, baseFrame.localVelocity,
                 localVelocity, worldVelocity);
    worldVelocity.timeStamp.setTimeStamp (localVelocity.timeStamp);
  }
  else {
    worldPosture = newPosture;
    worldVelocity = newVelocity;
    // Now get the local coordinagtes from the world ones
    // Position
    world2Local (baseFrame.worldPosture, worldPosture, localPosture);
    localPosture.timeStamp.setTimeStamp (worldPosture.timeStamp);
    // Velocity
    local2World (baseFrame.worldPosture, localPosture, baseFrame.localVelocity,
                 worldVelocity, localVelocity);
    localVelocity.timeStamp.setTimeStamp (worldVelocity.timeStamp);
  }
}



shared_ptr<Parameter> TrackedPosture::clone() const
{
  shared_ptr<TrackedPosture> newRobot (new TrackedPosture (*this));
  return newRobot;
}



string TrackedPosture::toString() const
{
  return worldPosture.toString() + localVelocity.toString();
}



//eof
