/**
 * @Filename TrackedPosture.hpp
 * @Description Vector Posture class header.
 * @Status Finished
 * @Version $Id: TrackedPosture.hpp 5750 2012-07-05 01:50:42Z jreis $
 * @Maintainer Hugo Costelha
 */

#ifndef _TRACKED_POSTURE_INFO_
#define _TRACKED_POSTURE_INFO_

#include <Data.hpp>
#include <Posture.hpp>
#include <VelocityPosture.hpp>
#include <FullPosture.hpp>



/**
 * @Class TrackedPosture TrackedPosture.hpp "TrackedPosture.hpp"
 * This class is used simply to aggregate all the information regarding the
 * a tracked (usually by a robot) object. This could be more elaborate, but time
 * is too short for that.
 * @Author Hugo Costelha
 */
class TrackedPosture : public Data
{
  public:
    /**
     * Constructor.
     * Everything set to 0/false.
     */
    TrackedPosture();
    
    /**
     * Constructor
     * @Argument newBaseFrame - the new base frame.
     * @Argument newPosture - posture in local or world coordinates of a given
     * object.
     * @Argument newVelocity - velocity in local or world coordinates of a given
     * object.
     * @Argument newSeeTrackedObject - if true, then the tracked object was
     * observed, otherwise it was etimated indirectly from other sources.
     * @Argument inLocalCoordinates - (optional, true by default) if true, the
     * posture and velocity coordinates of the tracked object are given in local
     * coordinates, otherwise they are given in world coordinates.
     */
    TrackedPosture (FullPosture newBaseFrame, Posture newPosture,
                    VelocityPosture newVelocity, bool newSeeTrackedObject,
                    bool inLocalCoordinates = true);
                    
    shared_ptr<Parameter> clone() const;
    string toString() const;
    
    FullPosture baseFrame; ///< Base frame info (usually the robot full posture)
    Posture localPosture; ///< Tracked posture in local coordinates.
    Posture worldPosture; ///< Tracked posture in world coordinates.
    VelocityPosture localVelocity; ///< Robot velocity in local coordinates.
    VelocityPosture worldVelocity; ///< Robot velocity in world coordinates.
    bool seeTrackedObject; ///< If true, the tracked objects was observed,
    ///< otherwise it was estimated from other sources.
};

#endif
