/**
 * @Filename Velocity2D.cpp
 * Velocity2D Data class implementation.
 * @Status Implementing
 * @Version $Id: Velocity2D.cpp 5661 2012-05-11 13:43:55Z jreis $
 * @Maintainer Hugo Costelha
 */
#include <Velocity2D.hpp>

#include <sstream>

#include <FrameTransformations2D.hpp>



Velocity2D::Velocity2D (void) : Data(),
  vX_ (0), vY_ (0), speed_ (0), azimuth_ (0), trustVx_ (1), trustVy_ (1),
  uncertainty_ (0)
{
}



Velocity2D::Velocity2D (string dataString) : Data(),
  vX_ (0), vY_ (0), speed_ (0), azimuth_ (0), trustVx_ (1), trustVy_ (1),
  uncertainty_ (0.0)
{
  const char* data;
  data = dataString.c_str();
  float inRho = 0, inAngle = 0;
  
  sscanf (data, "(%f, %f)", &inRho, &inAngle);
  setPolar (inRho, inAngle);
}



Velocity2D::Velocity2D (float speed, float azimuth, bool isRect) : Data(),
  vX_(), vY_(), speed_(), azimuth_(), trustVx_ (1), trustVy_ (1),
  uncertainty_ (0.0)
{
  if (isRect) {
    setVxVy (speed, azimuth);
  }
  else {
    setPolar (speed, azimuth);
  }
}



Velocity2D::~Velocity2D (void)
{
}



Speed Velocity2D::getVx() const
{
  return vX_;
}



Speed Velocity2D::getVy() const
{
  return vY_;
}



Speed Velocity2D::getSpeed() const
{
  return speed_;
}



Angle Velocity2D::getAzimuth() const
{
  return azimuth_;
}



void Velocity2D::setVxVy (Speed vX, Speed vY)
{
  vX_ = vX;
  vY_ = vY;
  updatePolar_();
}



void Velocity2D::setVx (Speed vX)
{
  setVxVy (vX, vY_);
}



void Velocity2D::setVy (Speed vY)
{
  setVxVy (vX_, vY);
}



void Velocity2D::setPolar (Speed speed, Angle azimuth)
{
  speed_ = speed;
  azimuth_ = azimuth;
  updateRect_();
}



void Velocity2D::setSpeed (Speed speed)
{
  setPolar (speed, azimuth_);
}



void Velocity2D::setAzimuth (Angle azimuth)
{
  setPolar (speed_, azimuth);
}



void Velocity2D::updateRect_()
{
  FrameTransformations2D::polar2Rect (speed_, azimuth_, &vX_, &vY_);
}



void Velocity2D::updatePolar_()
{
  FrameTransformations2D::rect2Polar (vX_, vY_, &speed_, &azimuth_);
}



void Velocity2D::setTrust (float trustVx, float trustVy)
{
  trustVx_ = trustVx;
  trustVy_ = trustVy;
}



float Velocity2D::getTrustVx() const
{
  return trustVx_;
}



float Velocity2D::getTrustVy() const
{
  return trustVy_;
}



void Velocity2D::setUncertainty (float value)
{
  uncertainty_ = value;
}



float Velocity2D::getUncertainty() const
{
  return uncertainty_;
}



shared_ptr<Parameter> Velocity2D::clone() const
{
  shared_ptr<Velocity2D> tmp (new Velocity2D (*this));
  return tmp;
}



string Velocity2D::toString() const
{
  stringstream ret;
  ret << "(" << getSpeed() << ", " << getAzimuth() << ")";
  
  return ret.str();
}
