#ifndef LOGGER_HPP
#define LOGGER_HPP

#include <vector>
#include <list>
#include <map>
#include <set>
#include <sstream>
#include <string>

#include <stdint.h>

#include <boost/foreach.hpp>
#define foreach BOOST_FOREACH



namespace mermaid
{
  namespace support
  {
    namespace logging
    {
      namespace LogAux
      {
        using std::string;
        using std::ostringstream;
        
        template <typename A>
        inline string toStr (const std::set<A>& theset)
        {
          ostringstream s;
          s << "[";
          bool isFirst = true;
          foreach (A t, theset) {
            if (!isFirst) {
              s << ",";
            }
            s << " " << t;
            isFirst = false;
          }
          s << " ]";
          return s.str();
        }
        
        template <>
        inline string toStr<uint8_t> (const std::set<uint8_t>& theset)
        {
          ostringstream s;
          s << "[";
          bool isFirst = true;
          foreach (uint8_t t, theset) {
            if (!isFirst) {
              s << ",";
            }
            s << " " << static_cast<int> (t);
            isFirst = false;
          }
          s << " ]";
          return s.str();
        }
        
        template <typename A>
        inline string toStr (const std::vector<A>& thevector)
        {
          ostringstream s;
          s << "[";
          bool isFirst = true;
          foreach (A t, thevector) {
            if (!isFirst) {
              s << ",";
            }
            s << " " << t;
            isFirst = false;
          }
          s << " ]";
          return s.str();
        }
        
        template <>
        inline string toStr<uint8_t> (const std::vector<uint8_t>& thevector)
        {
          ostringstream s;
          s << "[";
          bool isFirst = true;
          foreach (uint8_t t, thevector) {
            if (!isFirst) {
              s << ",";
            }
            s << " " << static_cast<int> (t);
            isFirst = false;
          }
          s << " ]";
          return s.str();
        }
        
        template <typename A>
        inline string toStr (const std::list<A>& thelist)
        {
          ostringstream s;
          s << "[";
          bool isFirst = true;
          foreach (A t, thelist) {
            if (!isFirst) {
              s << ",";
            }
            s << " " << t;
            isFirst = false;
          }
          s << " ]";
          return s.str();
        }
        
        template <typename A, typename B>
        inline string toStr (const std::map<A, B>& themap)
        {
          ostringstream s;
          s << "[";
          bool isFirst = true;
          typedef std::pair<A, B> pabtype;
          foreach (pabtype pab, themap) {
            if (!isFirst) {
              s << ",";
            }
            s << " (" << pab.first << ":" << pab.second << ")";
            isFirst = false;
          }
          s << " ]";
          return s.str();
        }
        
        inline string toStr (bool const& b)
        {
          return b ? "true" : "false";
        }
      }
    }
  }
}

#endif
