/**
 * @Filename MessageDataAcceleration.hpp
 * @Description Implementation of the MessageDataAcceleration class
 * @Status In progress
 * @Version $Id: MessageDataAcceleration.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_DATA_ACCELERATION_HPP
#define COMPONENTS_MESSAGE_DATA_ACCELERATION_HPP

#include <set>

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <socrob_ros/Pose2DStamped.h>



using std::set;



/**
 * @Class MessageDataAcceleration MessageDataAcceleration.hpp "MessageDataAcceleration.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageDataAcceleration
{
  public:
    static
    shared_ptr<MessageDataAcceleration>
    buildAndSend (float ax,
                  float ay,
                  float at,
                  TimeStamp const& ts) {
      static ros::Publisher pub = SocRobService::nh().advertise<socrob_ros::Pose2DStamped> ("acceleration", 1);
      
      socrob_ros::Pose2DStamped msg;
      msg.header.stamp = ros::Time::now();
      msg.pose2D.x = ax;
      msg.pose2D.y = ay;
      msg.pose2D.theta = at;
      pub.publish (msg);
      
      return shared_ptr<MessageDataAcceleration>();
    }
};



template <>
class Receiver<MessageDataAcceleration>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("acceleration", 1, &Receiver<MessageDataAcceleration>::accelerationCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    accelerationCallback (const socrob_ros::Pose2DStamped::ConstPtr& msg) {
      processAcceleration (msg->pose2D.x, msg->pose2D.y, msg->pose2D.theta, TimeStamp());
    }
    
    virtual
    void
    processAcceleration (float ax,
                         float ay,
                         float at,
                         TimeStamp const& ts) = 0;
                         
  private:
    ros::Subscriber sub;
};

#endif
