/**
 * @Filename MessageDataAllLinePoints.hpp
 * @Description Implementation of the MessageDataAllLinePoints class
 * @Status In progress
 * @Version $Id: MessageDataAllLinePoints.hpp 5936 2013-03-17 22:47:28Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_DATA_ALL_LINE_POINTS_HPP
#define COMPONENTS_MESSAGE_DATA_ALL_LINE_POINTS_HPP

#include <vector>

#include <Defines.hpp>

#include <Receiver.hpp>
#include <SocRobService.hpp>

#include <socrob_ros/FieldPointsStamped.h>



using std::set;



/**
 * @Class MessageDataAllLinePoints MessageDataAllLinePoints.hpp "MessageDataAllLinePoints.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageDataAllLinePoints
{
  public:
    static
    shared_ptr<MessageDataAllLinePoints>
    buildAndSend (shared_ptr<vector<point> const> const& linePoints,
                  shared_ptr<vector<point> const> const& obstaclePoints) {
      static ros::Publisher pub = SocRobService::nh().advertise<socrob_ros::FieldPointsStamped> ("sensors/line_points", 1);
      
      socrob_ros::FieldPointsStamped msg;
      
      foreach (point const & transition, *linePoints) {
        socrob_ros::Point2D p;
        p.x = transition.first;
        p.y = transition.second;
        msg.fieldPoints.linePoints.push_back (p);
      }
      foreach (point const & transition, *obstaclePoints) {
        socrob_ros::Point2D p;
        p.x = transition.first;
        p.y = transition.second;
        msg.fieldPoints.obstaclePoints.push_back (p);
      }
      
      pub.publish (msg);
      
      return shared_ptr<MessageDataAllLinePoints>();
    }
};



template <>
class Receiver<MessageDataAllLinePoints>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("sensors/line_points", 1, &Receiver<MessageDataAllLinePoints>::pointsCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    pointsCallback (const socrob_ros::FieldPointsStamped::ConstPtr& msg) {
      shared_ptr<vector<point> > linePoints (new vector<point>());
      shared_ptr<vector<point> > obstaclePoints (new vector<point>());
      
      foreach (socrob_ros::Point2D const & p, msg->fieldPoints.linePoints) {
        linePoints->push_back (make_pair (p.x, p.y));
      }
      foreach (socrob_ros::Point2D const & p, msg->fieldPoints.obstaclePoints) {
        obstaclePoints->push_back (make_pair (p.x, p.y));
      }
      
      processLinePoints (linePoints, obstaclePoints);
    }
    
    virtual
    void
    processLinePoints (shared_ptr<vector<point> const> const& linePoints,
                       shared_ptr<vector<point> const> const& obstaclePoints) = 0;
                       
  private:
    ros::Subscriber sub;
};

#endif
