/**
 * @Filename MessageDataBall.hpp
 * @Description Implementation of the MessageDataBall class
 * @Status In progress
 * @Version $Id: MessageDataBall.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_DATA_BALL_HPP
#define COMPONENTS_MESSAGE_DATA_BALL_HPP

#include <set>

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <socrob_ros/BallData.h>



using std::set;



/**
 * @Class MessageDataBall MessageDataBall.hpp "MessageDataBall.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageDataBall
{
  public:
    static
    shared_ptr<MessageDataBall>
    buildAndSend (float x,
                  float y,
                  float z,
                  float confidence) {
      static ros::NodeHandle n;
      static ros::Publisher pub = n.advertise<socrob_ros::BallData> ("ball_data", 1);
      
      socrob_ros::BallData ball;
      ball.position.x = x;
      ball.position.y = y;
      ball.position.z = z;
      ball.confidence = confidence;
      
      pub.publish (ball);
      
      return shared_ptr<MessageDataBall>();
    }
};



template <>
class Receiver<MessageDataBall>
{
  public:
    Receiver () :
      nh(),
      sub (nh.subscribe ("ball_data", 1, &Receiver<MessageDataBall>::ballCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    ballCallback (const socrob_ros::BallData::ConstPtr& ball) {
      processBallData (ball->position.x, ball->position.y, ball->position.z, ball->confidence);
    }
    
    virtual
    void
    processBallData (float x,
                     float y,
                     float z,
                     float confidence) = 0;
                     
  private:
    ros::NodeHandle nh;
    ros::Subscriber sub;
};

#endif
