/**
 * @Filename MessageDataCompass.hpp
 * @Description Implementation of the MessageDataCompass class
 * @Status In progress
 * @Version $Id: MessageDataCompass.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_DATA_COMPASS_HPP
#define COMPONENTS_MESSAGE_DATA_COMPASS_HPP

#include <set>

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <socrob_ros/CompassAngle.h>



using std::set;



/**
 * @Class MessageDataCompass MessageDataCompass.hpp "MessageDataCompass.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageDataCompass
{
  public:
    static
    shared_ptr<MessageDataCompass>
    buildAndSend (float angle,
                  TimeStamp const& ts) {
      static ros::Publisher pub = SocRobService::nh().advertise<socrob_ros::CompassAngle> ("compass_angle", 1);
      
      socrob_ros::CompassAngle msg;
      msg.header.stamp = ros::Time::now();
      msg.angle = angle;
      pub.publish (msg);
      
      return shared_ptr<MessageDataCompass>();
    }
};



template <>
class Receiver<MessageDataCompass>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("compass_angle", 1, &Receiver<MessageDataCompass>::compassCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    compassCallback (const socrob_ros::CompassAngle::ConstPtr& msg) {
      processCompassAngle (msg->angle, TimeStamp());
    }
    
    virtual
    void
    processCompassAngle (float angle,
                         TimeStamp const& ts) = 0;
                         
  private:
    ros::Subscriber sub;
};

#endif
