/**
 * @Filename MessageDataObstacles.hpp
 * @Description Implementation of the MessageDataObstacles class
 * @Status In progress
 * @Version $Id: MessageDataObstacles.hpp 5955 2013-04-09 08:41:34Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_DATA_OBSTACLES_HPP
#define COMPONENTS_MESSAGE_DATA_OBSTACLES_HPP

#include <set>

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <Obstacle.hpp>
#include <socrob_ros/ObstaclesStamped.h>



using std::set;



/**
 * @Class MessageDataObstacles MessageDataObstacles.hpp "MessageDataObstacles.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageDataObstacles
{
  public:
    static
    shared_ptr<MessageDataObstacles>
    buildAndSend (const shared_ptr<Obstacles>& obstacles) {
      static ros::Publisher pub = SocRobService::nh().advertise<socrob_ros::ObstaclesStamped> ("obstacles", 1);
      
      shared_ptr<socrob_ros::ObstaclesStamped> msg (new socrob_ros::ObstaclesStamped());
      msg->header.stamp = ros::Time::now();
      obstacles->rosMsg (msg->obstacles);
      pub.publish (msg);
      
      return shared_ptr<MessageDataObstacles>();
    }
};



template <>
class Receiver<MessageDataObstacles>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("obstacles", 1, &Receiver<MessageDataObstacles>::obstaclesCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    obstaclesCallback (const socrob_ros::ObstaclesStamped::ConstPtr& msg) {
      processObstacles (Obstacles::newFromRosMsg (msg->obstacles));
    }
    
    virtual
    void
    processObstacles (const shared_ptr<Obstacles>& obstacles) = 0;
    
  private:
    ros::Subscriber sub;
};

#endif
