/**
 * @Filename MessageDataOdometry.hpp
 * @Description Implementation of the MessageDataOdometry class
 * @Status In progress
 * @Version $Id: MessageDataOdometry.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_DATA_ODOMETRY_HPP
#define COMPONENTS_MESSAGE_DATA_ODOMETRY_HPP

#include <set>

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <socrob_ros/Pose2DStamped.h>



using std::set;



/**
 * @Class MessageDataOdometry MessageDataOdometry.hpp "MessageDataOdometry.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageDataOdometry
{
  public:
    static
    shared_ptr<MessageDataOdometry>
    buildAndSend (float dx,
                  float dy,
                  float dt,
                  TimeStamp const& ts) {
      static ros::Publisher pub = SocRobService::nh().advertise<socrob_ros::Pose2DStamped> ("odometry", 1);
      
      socrob_ros::Pose2DStamped msg;
      msg.header.stamp = ros::Time::now();
      msg.pose2D.x = dx;
      msg.pose2D.y = dy;
      msg.pose2D.theta = dt;
      pub.publish (msg);
      
      return shared_ptr<MessageDataOdometry>();
    }
};



template <>
class Receiver<MessageDataOdometry>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("odometry", 1, &Receiver<MessageDataOdometry>::odometryCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    odometryCallback (const socrob_ros::Pose2DStamped::ConstPtr& msg) {
      processOdometry (msg->pose2D.x, msg->pose2D.y, msg->pose2D.theta, TimeStamp());
    }
    
    virtual
    void
    processOdometry (float dx,
                     float dy,
                     float dt,
                     TimeStamp const& ts) = 0;
                     
  private:
    ros::Subscriber sub;
};

#endif
