/**
 * @Filename MessageDataShapes.hpp
 * @Description Implementation of the MessageDataShapes class
 * @Status In progress
 * @Version $Id: MessageDataShapes.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_DATA_SHAPES_HPP
#define COMPONENTS_MESSAGE_DATA_SHAPES_HPP

#include <set>

#include <Defines.hpp>

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <socrob_ros/FieldShapesStamped.h>



using std::set;



/**
 * @Class MessageDataShapes MessageDataShapes.hpp "MessageDataShapes.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageDataShapes
{
  public:
    static
    shared_ptr<MessageDataShapes>
    buildAndSend (shared_ptr<vector<line> const> const& line_ends,
                  shared_ptr<vector<line> const> const& boundary_ends,
                  shared_ptr<vector<point> const> const& big_circles,
                  shared_ptr<vector<point> const> const& small_circles,
                  int token,
                  TimeStamp const& ts) {
      static ros::Publisher pub = SocRobService::nh().advertise<socrob_ros::FieldShapesStamped> ("field_shapes", 1);
      
      socrob_ros::FieldShapesStamped msg;
      
      foreach (line const & l, *line_ends) {
        socrob_ros::LineSegment2D ls;
        ls.p0.x = l.first.first;
        ls.p0.y = l.first.second;
        ls.p1.x = l.second.first;
        ls.p1.y = l.second.second;
        msg.fieldShapes.lines.push_back (ls);
      }
      foreach (line const & l, *boundary_ends) {
        socrob_ros::LineSegment2D ls;
        ls.p0.x = l.first.first;
        ls.p0.y = l.first.second;
        ls.p1.x = l.second.first;
        ls.p1.y = l.second.second;
        msg.fieldShapes.boundaries.push_back (ls);
      }
      foreach (point const & c, *big_circles) {
        socrob_ros::Point2D circle;
        circle.x = c.first;
        circle.y = c.second;
        msg.fieldShapes.bigCircles.push_back (circle);
      }
      foreach (point const & c, *small_circles) {
        socrob_ros::Point2D circle;
        circle.x = c.first;
        circle.y = c.second;
        msg.fieldShapes.smallCircles.push_back (circle);
      }
      
      pub.publish (msg);
      
      return shared_ptr<MessageDataShapes>();
    }
};



template <>
class Receiver<MessageDataShapes>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("field_shapes", 1, &Receiver<MessageDataShapes>::shapesCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    shapesCallback (const socrob_ros::FieldShapesStamped::ConstPtr& msg) {
      shared_ptr<vector<line> > line_ends (new vector<line> ());
      shared_ptr<vector<line> > boundary_ends (new vector<line> ());
      shared_ptr<vector<point> > big_circles (new vector<point>());
      shared_ptr<vector<point> > small_circles (new vector<point>());
      
      foreach (socrob_ros::LineSegment2D const & line, msg->fieldShapes.lines) {
        line_ends->push_back (make_pair (point (line.p0.x, line.p0.y), point (line.p1.x, line.p1.y)));
      }
      foreach (socrob_ros::LineSegment2D const & line, msg->fieldShapes.boundaries) {
        boundary_ends->push_back (make_pair (point (line.p0.x, line.p0.y), point (line.p1.x, line.p1.y)));
      }
      foreach (socrob_ros::Point2D const & circle, msg->fieldShapes.bigCircles) {
        big_circles->push_back (point (circle.x, circle.y));
      }
      foreach (socrob_ros::Point2D const & circle, msg->fieldShapes.smallCircles) {
        small_circles->push_back (point (circle.x, circle.y));
      }
      
      processShapes (line_ends, boundary_ends, big_circles, small_circles, 0, TimeStamp());
    }
    
    virtual
    void
    processShapes (shared_ptr<vector<pair<point, point> > const> const& line_ends,
                   shared_ptr<vector<pair<point, point> > const> const& boundary_ends,
                   shared_ptr<vector<point> const> const& big_circles,
                   shared_ptr<vector<point> const> const& small_circles,
                   int token,
                   TimeStamp const& ts) = 0;
                   
  private:
    ros::Subscriber sub;
};

#endif
