/**
 * @Filename MessageEvent.hpp
 * @Description Implementation of the MessageEvent class
 * @Status In progress
 * @Version $Id: MessageEvent.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_EVENT_HPP
#define COMPONENTS_MESSAGE_EVENT_HPP

#include <set>

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <std_msgs/String.h>



using std::set;



/**
 * @Class MessageEvent MessageEvent.hpp "MessageEvent.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageEvent
{
  public:
    static
    shared_ptr<MessageEvent>
    buildAndSend (string const& event) {
      static ros::Publisher pub = SocRobService::nh().advertise<std_msgs::String> ("event", 100);
      
      std_msgs::String msg;
      msg.data = event;
      pub.publish (msg);
      
      return shared_ptr<MessageEvent>();
    }
};



template <>
class Receiver<MessageEvent>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("event", 100, &Receiver<MessageEvent>::eventCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    eventCallback (const std_msgs::String::ConstPtr& msg) {
      processEvent (msg->data);
    }
    
    virtual
    void
    processEvent (string const& event) = 0;
    
  private:
    ros::Subscriber sub;
};

#endif
