/**
 * @Filename MessageExecutePrimitives.hpp
 * @Description Implementation of the MessageExecutePrimitives class
 * @Status In progress
 * @Version $Id: MessageExecutePrimitives.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_EXECUTE_PRIMITIVES_HPP
#define COMPONENTS_MESSAGE_EXECUTE_PRIMITIVES_HPP

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <socrob_ros/StringArray.h>



/**
 * @Class MessageExecutePrimitives MessageExecutePrimitives.hpp "MessageExecutePrimitives.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageExecutePrimitives
{
  public:
    static
    shared_ptr<MessageExecutePrimitives>
    buildAndSend (string const&,
                  const shared_ptr<set<string> >& primitives) {
      static ros::Publisher pub = SocRobService::nh().advertise<socrob_ros::StringArray> ("execute_primitives", 100, true);
      
      socrob_ros::StringArray msg;
      foreach (string const & p, *primitives) {
        msg.data.push_back (p);
      }
      pub.publish (msg);
      
      return shared_ptr<MessageExecutePrimitives>();
    }
};



template <>
class Receiver<MessageExecutePrimitives>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("execute_primitives", 100, &Receiver<MessageExecutePrimitives>::executePrimitivesCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    executePrimitivesCallback (const socrob_ros::StringArray::ConstPtr& msg) {
      shared_ptr<set<string> > tmp = shared_ptr<set<string> > (new set<string>());
      foreach (string const & p, msg->data) {
        tmp->insert (p);
      }
      executePrimitives (tmp);
    }
    
    virtual
    void
    executePrimitives (const shared_ptr<set<string> >& primitives) = 0;
    
  private:
    ros::Subscriber sub;
};

#endif
