/**
 * @Filename MessageKick.hpp
 * @Description Implementation of the MessageKick class
 * @Status In progress
 * @Version $Id: MessageKick.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_KICK_HPP
#define COMPONENTS_MESSAGE_KICK_HPP

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <std_msgs/Float32.h>



/**
 * @Class MessageKick MessageKick.hpp "MessageKick.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageKick
{
  public:
    static
    shared_ptr<MessageKick>
    buildAndSend (int strengh) {
      static ros::Publisher pub = SocRobService::nh().advertise<std_msgs::Float32> ("kick", 1);
      
      std_msgs::Float32 msg;
      msg.data = strengh;
      pub.publish (msg);
      
      return shared_ptr<MessageKick>();
    }
};



template <>
class Receiver<MessageKick>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("kick", 1, &Receiver<MessageKick>::kickCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    kickCallback (const std_msgs::Float32::ConstPtr& msg) {
      kick (msg->data);
    }
    
    virtual
    void
    kick (int strengh) = 0;
    
  private:
    ros::Subscriber sub;
};

#endif
