/**
 * @Filename MessageRegisterInPredicates.hpp
 * @Description Implementation of the MessageRegisterInPredicates class
 * @Status In progress
 * @Version $Id: MessageRegisterInPredicates.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_REGISTER_IN_PREDICATES_HPP
#define COMPONENTS_MESSAGE_REGISTER_IN_PREDICATES_HPP

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <socrob_ros/RegisterInPredicates.h>



/**
 * @Class MessageRegisterInPredicates MessageRegisterInPredicates.hpp "MessageRegisterInPredicates.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageRegisterInPredicates
{
  public:
    static
    shared_ptr<MessageRegisterInPredicates>
    buildAndSend (const shared_ptr<set<string> >& predicates) {
      ros::service::waitForService ("register_in_predicates", 10000);
      
      socrob_ros::RegisterInPredicates srv;
      srv.request.predicates.insert (srv.request.predicates.end(), predicates->begin(), predicates->end());
      if (! ros::service::call ("register_in_predicates", srv)) {
        ROS_FATAL_STREAM (ros::this_node::getName() << ": Could not register in predicates!");
      }
      
      return shared_ptr<MessageRegisterInPredicates>();
    }
};



template <>
class Receiver<MessageRegisterInPredicates>
{
  public:
    virtual
    ~Receiver() {}
    
    bool
    regpredCallback (socrob_ros::RegisterInPredicates::Request&  req, socrob_ros::RegisterInPredicates::Response& res) {
      shared_ptr<set<string> > tmp = shared_ptr<set<string> > (new set<string>());
      tmp->insert (req.predicates.begin(), req.predicates.end());
      registerInPredicates ("dummy", tmp);
      
      return true;
    }
    
    
    virtual
    void
    registerInPredicates (string const& aoName,
                          const shared_ptr<set<string> >& predicates) = 0;
                          
  protected:
    void startServiceRegisterInPredicates() {
      service = SocRobService::nh().advertiseService ("register_in_predicates", &Receiver<MessageRegisterInPredicates>::regpredCallback, this);
    }
    
  private:
    ros::ServiceServer service;
};

#endif
