/**
 * @Filename MessageSetRobotFinalVelocity.hpp
 * @Description Implementation of the MessageSetRobotFinalVelocity class
 * @Status In progress
 * @Version $Id: MessageSetRobotFinalVelocity.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_SET_ROBOT_FINAL_VELOCITY_HPP
#define COMPONENTS_MESSAGE_SET_ROBOT_FINAL_VELOCITY_HPP

#include <Receiver.hpp>

#include <SocRobService.hpp>
#include <ControlInput.hpp>
#include <HelperMessages.hpp>

#include <socrob_ros/ControlInput.h>



/**
 * @Class MessageSetRobotFinalVelocity MessageSetRobotFinalVelocity.hpp "MessageSetRobotFinalVelocity.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageSetRobotFinalVelocity
{
  public:
    static
    shared_ptr<MessageSetRobotFinalVelocity>
    buildAndSend (float vx,
                  float vy,
                  float w,
                  CONTROL_PRIORITY vPrio = BALANCED) {
      ControlInput u;
      u.setType (VELOCITY);
      u.setLinearX (vx);
      u.setLinearY (vy);
      u.setAngular (w);
      u.setPriority (vPrio);
      
      static ros::Publisher pub = SocRobService::nh().advertise<socrob_ros::ControlInput> ("control_input_final", 1, true);
      socrob_ros::ControlInput msg;
      u.rosMsg (msg);
      pub.publish (msg);
      
      return shared_ptr<MessageSetRobotFinalVelocity>();
    }
};



template <>
class Receiver<MessageSetRobotFinalVelocity>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("control_input_final", 1, &Receiver<MessageSetRobotFinalVelocity>::controlFinalCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    controlFinalCallback (const socrob_ros::ControlInput::ConstPtr& msg) {
      ControlInput u;
      u.fromRosMsg (*msg);
      switch (msg->type) {
        case socrob_ros::ControlInput::VELOCITY:
          setRobotVelocity (u.getLinearX(), u.getLinearY(), u.getAngular(), u.getPriority());
          break;
        case socrob_ros::ControlInput::ACCELERATION:
          setRobotAcceleration (u.getLinearX(), u.getLinearY(), u.getAngular(), u.getPriority());
          break;
      }
    }
    
    virtual
    void
    setRobotVelocity (float vx,
                      float vy,
                      float w,
                      CONTROL_PRIORITY prio) = 0;
                      
    virtual
    void
    setRobotAcceleration (float ax,
                          float ay,
                          float alpha,
                          CONTROL_PRIORITY vPrio = BALANCED) {}
                          
  private:
    ros::Subscriber sub;
};

#endif
