/**
 * @Filename MessageStartBehaviors.hpp
 * @Description Implementation of the MessageStartBehaviors class
 * @Status In progress
 * @Version $Id: MessageStartBehaviors.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_START_BEHAVIORS_HPP
#define COMPONENTS_MESSAGE_START_BEHAVIORS_HPP

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <socrob_ros/StringArray.h>



/**
 * @Class MessageStartBehaviors MessageStartBehaviors.hpp "MessageStartBehaviors.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageStartBehaviors
{
  public:
    static
    shared_ptr<MessageStartBehaviors>
    buildAndSend (string const&,
                  shared_ptr<set<string> > const& tasks) {
      static ros::Publisher pub = SocRobService::nh().advertise<socrob_ros::StringArray> ("start_behaviors", 100, true);
      
      socrob_ros::StringArray msg;
      foreach (string const & t, *tasks) {
        msg.data.push_back (t);
      }
      pub.publish (msg);
      
      return shared_ptr<MessageStartBehaviors>();
    }
    
    static
    shared_ptr<MessageStartBehaviors>
    buildAndSend (string const&,
                  string const& task) {
      static ros::Publisher pub = SocRobService::nh().advertise<socrob_ros::StringArray> ("start_behaviors", 100);
      
      socrob_ros::StringArray msg;
      msg.data.push_back (task);
      pub.publish (msg);
      
      return shared_ptr<MessageStartBehaviors>();
    }
};



template <>
class Receiver<MessageStartBehaviors>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("start_behaviors", 100, &Receiver<MessageStartBehaviors>::startBehaviorsCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    startBehaviorsCallback (const socrob_ros::StringArray::ConstPtr& msg) {
      shared_ptr<set<string> > tmp = shared_ptr<set<string> > (new set<string>());
      foreach (string const & t, msg->data) {
        tmp->insert (t);
      }
      startBehaviors (tmp);
    }
    
    virtual
    void
    startBehaviors (shared_ptr<set<string> > const& tasks) = 0;
    
  private:
    ros::Subscriber sub;
};

#endif
