/**
 * @Filename MessageStartPrimitive.hpp
 * @Description Implementation of the MessageStartPrimitive class
 * @Status In progress
 * @Version $Id: MessageStartPrimitive.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_START_PRIMITIVE_HPP
#define COMPONENTS_MESSAGE_START_PRIMITIVE_HPP

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <std_msgs/String.h>



/**
 * @Class MessageStartPrimitive MessageStartPrimitive.hpp "MessageStartPrimitive.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageStartPrimitive
{
  public:
    static
    shared_ptr<MessageStartPrimitive>
    buildAndSend (string const&,
                  string const& action) {
      static ros::Publisher pub = SocRobService::nh().advertise<std_msgs::String> ("start_primitive", 100, true);
      
      std_msgs::String msg;
      msg.data = action;
      pub.publish (msg);
      
      return shared_ptr<MessageStartPrimitive>();
    }
};



template <>
class Receiver<MessageStartPrimitive>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("start_primitive", 100, &Receiver<MessageStartPrimitive>::startPrimitiveCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    startPrimitiveCallback (const std_msgs::String::ConstPtr& msg) {
      startPrimitive (msg->data);
    }
    
    virtual
    void
    startPrimitive (string const& action) = 0;
    
  private:
    ros::Subscriber sub;
};

#endif
