/**
 * @Filename MessageStartRoles.hpp
 * @Description Implementation of the MessageStartRoles class
 * @Status In progress
 * @Version $Id: MessageStartRoles.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_START_ROLES_HPP
#define COMPONENTS_MESSAGE_START_ROLES_HPP

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <socrob_ros/StringArray.h>



/**
 * @Class MessageStartRoles MessageStartRoles.hpp "MessageStartRoles.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageStartRoles
{
  public:
    static
    shared_ptr<MessageStartRoles>
    buildAndSend (string const&,
                  const shared_ptr<set<string> >& tasks) {
      static ros::Publisher pub = SocRobService::nh().advertise<socrob_ros::StringArray> ("start_roles", 100, true);
      
      socrob_ros::StringArray msg;
      foreach (string const & t, *tasks) {
        msg.data.push_back (t);
      }
      pub.publish (msg);
      
      return shared_ptr<MessageStartRoles>();
    }
    
    static
    shared_ptr<MessageStartRoles>
    buildAndSend (string const&,
                  string const& task) {
      static ros::Publisher pub = SocRobService::nh().advertise<socrob_ros::StringArray> ("start_roles", 100);
      
      socrob_ros::StringArray msg;
      msg.data.push_back (task);
      pub.publish (msg);
      
      return shared_ptr<MessageStartRoles>();
    }
};



template <>
class Receiver<MessageStartRoles>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("start_roles", 100, &Receiver<MessageStartRoles>::startRolesCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    startRolesCallback (const socrob_ros::StringArray::ConstPtr& msg) {
      shared_ptr<set<string> > tmp = shared_ptr<set<string> > (new set<string>());
      foreach (string const & t, msg->data) {
        tmp->insert (t);
      }
      startRoles (tmp);
    }
    
    virtual
    void
    startRoles (const shared_ptr<set<string> >& tasks) = 0;
    
  private:
    ros::Subscriber sub;
};

#endif
