/**
 * @Filename MessageTestPrimitive.hpp
 * @Description Implementation of the MessageTestPrimitive class
 * @Status In progress
 * @Version $Id: MessageTestPrimitive.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_TEST_PRIMITIVE_HPP
#define COMPONENTS_MESSAGE_TEST_PRIMITIVE_HPP

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <std_msgs/String.h>



/**
 * @Class MessageTestPrimitive MessageTestPrimitive.hpp "MessageTestPrimitive.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageTestPrimitive
{
  public:
    static
    shared_ptr<MessageTestPrimitive>
    buildAndSend (string const&,
                  string const& action) {
      static ros::Publisher pub = SocRobService::nh().advertise<std_msgs::String> ("test_primitive", 100, true);
      
      std_msgs::String msg;
      msg.data = action;
      pub.publish (msg);
      
      return shared_ptr<MessageTestPrimitive>();
    }
};



template <>
class Receiver<MessageTestPrimitive>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("test_primitive", 100, &Receiver<MessageTestPrimitive>::testPrimitiveCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    testPrimitiveCallback (const std_msgs::String::ConstPtr& msg) {
      testPrimitive (msg->data);
    }
    
    virtual
    void
    testPrimitive (string const& action) = 0;
    
  private:
    ros::Subscriber sub;
};

#endif
