/**
 * @Filename MessageUpdatePredicates.hpp
 * @Description Implementation of the MessageUpdatePredicates class
 * @Status In progress
 * @Version $Id: MessageUpdatePredicates.hpp 5866 2012-12-14 01:07:05Z jreis $
 * @Maintainer Joao Reis
 */

#ifndef COMPONENTS_MESSAGE_UPDATE_PREDICATES_HPP
#define COMPONENTS_MESSAGE_UPDATE_PREDICATES_HPP

#include <Receiver.hpp>

#include <SocRobService.hpp>

#include <socrob_ros/PredicatesUpdate.h>



/**
 * @Class MessageUpdatePredicates MessageUpdatePredicates.hpp "MessageUpdatePredicates.hpp"
 * Message.
 * @Author Joao Reis
 */
class MessageUpdatePredicates
{
  public:
    static
    shared_ptr<MessageUpdatePredicates>
    buildAndSend (string const&,
                  const shared_ptr<map<string, bool> >& predicates) {
      static ros::Publisher pub = SocRobService::nh().advertise<socrob_ros::PredicatesUpdate> ("predicates", 100, true);
      
      socrob_ros::PredicatesUpdate msg;
      typedef pair<string, bool> psb;
      foreach (psb const & p, *predicates) {
        if (p.second) {
          msg.predicatesTrue.push_back (p.first);
        }
        else {
          msg.predicatesFalse.push_back (p.first);
        }
      }
      pub.publish (msg);
      
      return shared_ptr<MessageUpdatePredicates>();
    }
};



template <>
class Receiver<MessageUpdatePredicates>
{
  public:
    Receiver () :
      sub (SocRobService::nh().subscribe ("predicates", 100, &Receiver<MessageUpdatePredicates>::updatePredicatesCallback, this))
    {}
    
    virtual
    ~Receiver() {}
    
    void
    updatePredicatesCallback (const socrob_ros::PredicatesUpdate::ConstPtr& msg) {
      shared_ptr<map<string, bool> > tmp = shared_ptr<map<string, bool> > (new map<string, bool>());
      foreach (string const & p, msg->predicatesTrue) {
        (*tmp) [p] = true;
      }
      foreach (string const & p, msg->predicatesFalse) {
        (*tmp) [p] = false;
      }
      updatePredicates (tmp);
    }
    
    virtual
    void
    updatePredicates (const shared_ptr<map<string, bool> >& predicates) = 0;
    
  private:
    ros::Subscriber sub;
};

#endif
