/**
 * @Filename Parameter.hpp
 * @Description Support Parameter declaration
 * @Status Work in Progress
 * @Version $Id: Parameter.hpp
 * @Maintainer nmsra (nmsra@mega.ist.utl.pt)
 */

#ifndef _SUPPORT_PARAMETER_HPP_
#define _SUPPORT_PARAMETER_HPP_

#include <string>
#include <iostream>

#include <boost/foreach.hpp>
#define foreach BOOST_FOREACH
#include <boost/shared_ptr.hpp>
#include <boost/pointer_cast.hpp>
using boost::shared_ptr;
using boost::static_pointer_cast;
using boost::dynamic_pointer_cast;



using namespace std;



/**
 * @Class Parameter Parameter.hpp "Parameter.hpp"
 * This class is an abstration of parameters that can be passed between ActiveObjects. All parameters (like Integer, String, Float, ActiveObject, ect) must to inherit of class Parameter.
 * @Author Nelson Ramos
 */
//template <class Child>
class Parameter
{
  public:
  
    /**
     * Constructor.
     * This method creates a new Parameter
     * @Author Nelson Ramos
     */
    Parameter();
    
    /**
     * Destructor.
     * This method makes deinitializations for objects of this class.
     * @Author Nelson Ramos
     */
    virtual ~Parameter();
    
    /**
     * This method know how to clone the parameter
     * @Author Nelson Ramos
     */
    virtual shared_ptr<Parameter> clone() const = 0;
    
    /**
     * cast operations
     * return the identification of parameter
     * @Author Nelson Ramos
     */
    virtual string toString() const = 0;
    
    /**
     * This method change the clone permition of this parameter
     * @Argument value - ?
     * @Author Nelson Ramos
     */
    void setClonable (bool value);
    
  protected:
    /**
     * Overload the copy constructor, so that we control the copies are made
     */
    Parameter (const Parameter& oldParameter);
    
    /**
     * Overload the = operator, so that we control how copies are made
     */
    Parameter& operator= (const Parameter& oldParameter);
    
    
  private:
    bool clonable_; ///< True if the object can be cloned
};

#endif
