#include "camera_watchdog.h"

#include <cv_bridge/cv_bridge.h>

#include <std_msgs/Bool.h>



using namespace std;
using namespace ros;



CameraWatchdog::
CameraWatchdog () :
  last_time_ (TIME_MIN),
  ok_ (true), // No good reason for this. Only this way, the error will not get printed if camera is ok as it should.
  nh_(),
  it_ (nh_),
  ok_pub_ (nh_.advertise<std_msgs::Bool> ("watchdogs/camera", 1, true)),
  image_sub_ (it_.subscribe ("devices/topcam/image_raw", 1, &CameraWatchdog::imageCallback, this)),
  timer_ (nh_.createTimer (ros::Duration (0.1), &CameraWatchdog::timer, this))
{
  updateParams();
  
  std_msgs::Bool msg;
  msg.data = ok_;
  ok_pub_.publish (msg);
}



void
CameraWatchdog::
updateParams()
{
  if (! ros::param::getCached ("~max_period", max_period_)) {
    max_period_ = 0.5;
    ros::param::set ("~max_period", max_period_);
  }
}



void
CameraWatchdog::
imageCallback (const sensor_msgs::ImageConstPtr& image)
{
  last_time_ = Time::now();
  updateOk (true);
}



void
CameraWatchdog::
timer (ros::TimerEvent const& timer_event)
{
  updateParams();
  
  if (timer_event.current_real - last_time_ > Duration (max_period_)) {
    updateOk (false);
  }
  else {
    updateOk (true);
  }
}



void
CameraWatchdog::
updateOk (bool ok)
{
  if (ok == ok_) {
    return;
  }
  ok_ = ok;
  
  ROS_ERROR_STREAM ("Camera is " + string (ok ? "ON" : "OFF"));
  
  std_msgs::Bool msg;
  msg.data = ok;
  ok_pub_.publish (msg);
}
