#include "show_stopper.h"



using namespace std;
using namespace ros;



ShowStopper::
ShowStopper () :
  camera_ok_ (false),
  temperature_ok_ (false),
  all_ok_ (false),
  nh_(),
  all_ok_pub_ (nh_.advertise<std_msgs::Bool> ("watchdogs/all_ok", 1, true)),
  camera_sub_ (nh_.subscribe ("watchdogs/camera", 1, &ShowStopper::cameraCallback, this)),
  temperature_sub_ (nh_.subscribe ("watchdogs/temperature", 1, &ShowStopper::temperatureCallback, this)),
  timer_ (nh_.createTimer (ros::Duration (1.0), &ShowStopper::timer, this))
{
  updateParams();
  
  std_msgs::Bool msg;
  msg.data = all_ok_;
  all_ok_pub_.publish (msg);
}



void
ShowStopper::
updateParams()
{
  if (! ros::param::getCached ("~ignore_camera", ignore_camera_)) {
    ignore_camera_ = false;
    ros::param::set ("~ignore_camera", ignore_camera_);
  }
  
  if (! ros::param::getCached ("~ignore_temperature", ignore_temperature_)) {
    ignore_temperature_ = false;
    ros::param::set ("~ignore_temperature", ignore_temperature_);
  }
}



void
ShowStopper::
cameraCallback (std_msgs::Bool const& ok)
{
  camera_ok_ = ok.data;
  updateOk();
}



void
ShowStopper::
temperatureCallback (std_msgs::Bool const& ok)
{
  temperature_ok_ = ok.data;
  updateOk();
}



void
ShowStopper::
timer (ros::TimerEvent const&)
{
  updateOk();
}



void
ShowStopper::
updateOk ()
{
  updateParams();
  
  bool all_ok = true;
  all_ok = all_ok && (ignore_camera_ || (camera_ok_ && (camera_sub_.getNumPublishers() > 0)));
  all_ok = all_ok && (ignore_temperature_ || (temperature_ok_ && (temperature_sub_.getNumPublishers() > 0)));
  
  if (all_ok == all_ok_) {
    return;
  }
  all_ok_ = all_ok;
  
  ROS_ERROR_STREAM ("Show Stopper is " + string (all_ok ? "ALL OK" : "NOT OK"));
  
  std_msgs::Bool msg;
  msg.data = all_ok;
  all_ok_pub_.publish (msg);
}
