#include "temperature_watchdog.h"

#include <algorithm>

#include <std_msgs/Bool.h>



using namespace std;
using namespace ros;



TemperatureWatchdog::
TemperatureWatchdog () :
  ok_ (true), // Temperature sensors might be disconnected or temperature might be in the no-update interval
  nh_(),
  ok_pub_ (nh_.advertise<std_msgs::Bool> ("watchdogs/temperature", 1, true)),
  temp_sub_ (nh_.subscribe ("temperature", 1, &TemperatureWatchdog::temperatureCallback, this))
{
  updateParams();
  
  std_msgs::Bool msg;
  msg.data = ok_;
  ok_pub_.publish (msg);
}



void
TemperatureWatchdog::
updateParams()
{
  if (! param::getCached ("~stop_temp", stop_temp_)) {
    stop_temp_ = 55;
    param::set ("~stop_temp", stop_temp_);
  }
  
  if (! param::getCached ("~move_again_temp", move_again_temp_)) {
    move_again_temp_ = 50;
    param::set ("~move_again_temp", move_again_temp_);
  }
}



void
TemperatureWatchdog::
temperatureCallback (const socrob_ros::TemperatureStamped::ConstPtr& msg)
{
  updateParams();
  
  int max_temp = std::max (msg->temp1, std::max (msg->temp2, msg->temp3));
  
  if (max_temp > stop_temp_) {
    updateOk (false);
  }
  else if (max_temp < move_again_temp_) {
    updateOk (true);
  }
}



void
TemperatureWatchdog::
updateOk (bool ok)
{
  if (ok == ok_) {
    return;
  }
  ok_ = ok;
  
  ROS_ERROR_STREAM ("Temperature is " + string (ok ? "OK" : " TOO HIGH"));
  
  std_msgs::Bool msg;
  msg.data = ok;
  ok_pub_.publish (msg);
}
